/* File    : Ac_base.cc
 * Includes: functions for the Base actuator class
 */

#include "decls.h"
#include "Ac_base.h"
#include "interface.h"
// not needed in final version!
#include <Ma/Ma_me.h>

enum { GOTOBALL = 1, TIME, TURN, DASH, NO_ACTION } ac_base_traces ;

const float AC_BASE_MIN_TURN = 0.05;
const float AC_BASE_MIN_SPD_DIFF = 0.1;

#include <fstream.h>

extern Interface interface;

/* constructor function */
Ac_base::Ac_base ()
{
    
    my_id = AC_BASE_ID;
    inhibited = NO;
    
    model.reset ();
    
    prev_action_type = AC_BASE_DRIVE;
    
    curr_time = 0;
//    lost_impulse = 0;
        
    last_act = AC_BASE_NONE;
    last_turn_dir = 0;
    
    prev_cmd.action = curr_cmd.action = AC_BASE_NONE;
    prev_cmd.target_speed = curr_cmd.target_speed = 0.0;
    prev_cmd.target_d_angle = curr_cmd.target_d_angle = 0.0;
    prev_cmd.time = curr_cmd.time = 0;
    
    if(Cons2::debug_level_startup()) cerr << "Ac_base: CONSTRUCTOR executed" << endl;
}

Ac_base::~Ac_base ()
{
     /* DEBUG */
     if(Cons2::debug_level_exit()) cerr << "Ac_base: DESTRUCTOR executed" << endl;
}

void Ac_base::Init(EmptySamba *sambapointer)
{
    samba = sambapointer;
    arbiter = (Arbiter*)samba->get_pointer( ARBITER_ID );
    sensor  = (Sensor*) samba->get_pointer( SENSOR_ID );
}

void Ac_base::invalidate_all()
{
    // WHEN SOME DATA IS ADDED TO THIS MODULE THIS FUNCTION MUST BE
    // FINISHED!!!!!!
    prev_cmd = curr_cmd;
    
    curr_cmd.target_speed = 0.0;
    curr_cmd.target_d_angle = 0.0;
    curr_cmd.action = AC_BASE_NONE;
    
    // Maybe this works well. Command from the previous cycle is always sent
    // to other modules by read command. 
}

void Ac_base::no_trigger_read(Base_actuator &to)
{
//     check_executed();
     to = prev_cmd;
}

int Ac_base::transform ()
{
    // read move command from the arbiter
    arbiter->read(move_cmd);
    // read sensors to get current speed of the robot
    sensor->read_sense_body(sense_body);
    if(sense_body.valid) curr_spd = sense_body.speed;// /Cons2::player_decay();
      
    if(!move_cmd.valid){
        last_act = AC_BASE_NONE;
        return 0;
    }
          
     // turn if turn command greater than minimum command and last action was
     // not turn, OR turn if command is creater than 45 degrees even 
     // if last command was turn
     if( fabs(move_cmd.dir) > AC_BASE_MIN_TURN
         && (last_act != AC_BASE_TURN 
             //|| move_cmd.dir * (float)last_turn_dir > 0.0
             )){
         // calculate how big impulse is needed to satisfy turning command    
         float impulse = move_cmd.dir * (1.0 + 5.0*Cons2::player_decay()*curr_spd);
                  
         if(impulse > 180.0) impulse = 180.0;
         else if(impulse < -180.0) impulse = -180.0;
         
         curr_cmd.target_d_angle = move_cmd.dir;
         
         interface.command_turn(impulse);
         last_act = curr_cmd.action = AC_BASE_TURN;
         
/*         lost_impulse += (move_cmd.spd-Cons2::player_decay()*move_cmd.spd)/ 
             Cons2::dash_power_rate();
         if(lost_impulse > 100) lost_impulse = 100;
         else if(lost_impulse < 0) lost_impulse = 0;*/
         
         if(move_cmd.dir < 0) last_turn_dir = -1;
         else last_turn_dir = 1;
     }
     // drive
     else {
         if( fabs(move_cmd.dir) > AC_BASE_MIN_TURN) 
                      
         // this must be here to allow turning at next cycle even if 
         // moving was not done
         last_act = AC_BASE_NONE;
         if( move_cmd.spd != 0.0 ){
             if(move_cmd.spd < Cons2::player_decay()*curr_spd && move_cmd.spd > 0.0){
                 return 0;
             }
             
             float impulse;
             impulse = (move_cmd.spd-Cons2::player_decay()*move_cmd.spd)/ 
                 Cons2::dash_power_rate();
/*             if(impulse >0) impulse += lost_impulse;
             lost_impulse = 0;
             if(impulse > 100){
                 if(last_act != AC_BASE_DRIVE) lost_impulse = impulse - 100;
                 impulse = 100;
                 }*/
             
             interface.command_dash(impulse);
             
             last_act = curr_cmd.action = AC_BASE_DRIVE;
             curr_cmd.target_speed = move_cmd.spd;
         } else {
             
         }
     }
     return 0;
}
