// -*- mode:C++ -*-

// file    : Ac_base.h
// includes: class declaration for the Base arbiter and actuator
// 

#ifndef _AC_BASE_H
#define _AC_BASE_H 1
// action types executed by the base
enum Ac_base_Actions { AC_BASE_NONE, AC_BASE_DRIVE = 1, AC_BASE_TURN };

// 
struct Base_actuator
{
     int time;
     float target_speed;
     float target_d_angle;
     Ac_base_Actions action;
};
class Ac_base;


#include "Module.h"
#include "Sensor.h"
#include <Ma/Ma_ball.h>
#include <Base_model.h>
#include "Arbiter.h"


class Ac_base: public Module {
public:
     Ac_base();
     ~Ac_base();
     virtual void reset () {;}
//     virtual void execute();
     void Init(EmptySamba *sambapointer);
     
    // functions to serve other modules
    // no_trigger means that module is not executed by calling this
    // read function
    void no_trigger_read(Base_actuator &to);
     
protected:
    void arbitrate();
    FLAG valid_conds() { return YES; };
    void invalidate_all();
    int transform();
    
    Arbiter *arbiter;
    Sensor *sensor;
    
    Move_cmd move_cmd;
    Sense_body sense_body;
    

    // class for modeling physics of robot's environment
    Base_model model;
    
    Ac_base_Actions prev_action_type; // turn or drive
        
    float  lost_impulse;
    float curr_spd;
    
    Base_actuator prev_cmd, curr_cmd;
    Ac_base_Actions last_act;
    int last_turn_dir;
};

#endif

// end of file



