/* File    : Ac_kick.cc
 * Includes: functions for the Kick actuator class
 */

#include "decls.h"
#include "Ac_kick.h"

#include "interface.h"
// Interface is declared to be a global variable!!!
extern Interface interface;

/* constructor function */
Ac_kick::Ac_kick ()
{
  my_id = AC_KICK_ID;
  inhibited = NO;

  curr_kicks.valid  = prev_kicks.valid = 0;
  if(Cons2::debug_level_startup()) cerr << "Ac_kick: CONSTRUCTOR executed" << endl;
}


Ac_kick::~Ac_kick ()
{
    if(Cons2::debug_level_exit()) cerr << "Ac_kick: DESTRUCTOR executed" << endl;
}

void Ac_kick::Init(EmptySamba *sambapointer)
{
    samba = sambapointer;
    arbiter = (Arbiter*)samba->get_pointer( ARBITER_ID );
}

// input:  distance to target
//         velocity at target
// output: impulse 
float Ac_kick::kick_impulse(float dist, float end_spd)
{
     float v;

     v = dist*(1-Cons2::ball_decay()) + end_spd; // velocity at the beginning
     return v/Cons2::kick_power_rate(); // impulse to gain the velocity
}

void Ac_kick::no_trigger_read(Kick_actuator &to)
{
    to = prev_kicks;
}

int Ac_kick::catched()
{
    return prev_hold;
}

void Ac_kick::invalidate_all()
{
    prev_kicks = curr_kicks;
    curr_kicks.valid = 0;
    kick.valid = 0;

    prev_hold = curr_hold;
    curr_hold = 0;
}

int Ac_kick::transform ()
{
    Ball_marker ball;
    Me_marker me;
    
    ((Ma_ball*)(samba->get_pointer(MA_BALL_ID)))->read(ball);
    ((Ma_me*)(samba->get_pointer(MA_ME_ID)))->read_exec(me);

    arbiter->read(hold_cmd);
    if(hold_cmd.valid){
        interface.command_catch(hold_cmd.dir);
        curr_hold = 1;
#ifndef NOPRINTS
        cout << "Goalkeeper holding!"<<endl;
#endif
        return 0;
    } 

    arbiter->read(kick);
    if(kick.valid){
        interface.command_kick(kick.imp.get_len(), 
                               kick.imp.get_dir());
        curr_kicks.target_g_vel = kick.g_vel;
        curr_kicks.time = curr_time;
        curr_kicks.valid = 1;
    }
    return 0;
    
}

/* end of file */
