// -*- mode:C++ -*-                                     
// File:   Actionmap.h
// Author: Antti Tikanmki, Jussi Pajala
// Contact: sunday@ee.oulu.fi, jussip@ee.oulu.fi
// This file declares Actionmap class
// This class is mother and father of all Actionmap classes.
#ifndef ACTIONMAP_H
#define ACTIONMAP_H
#include <Am/Polargrid.h>
#include <iostream.h>
#include "Vector.h"
#include "Cons2.h"

enum save_what {WGT, WGT_OBJ, WGT_OBJ_BEST};

class Actionmap {
protected:
     float obj_dir;
     float obj_dist;
     float obj_spd;
     float obj_vel_dir;
     static float CUR_MAX_SPEED;
     Polargrid amap;
     static float *discr_cos; // to save discrete cosine

     void project_dyn(int dir, int row, int col,float reach_time);
     void project_stat(int row, int col);
     int fill_collision_angle(int row, int col, float reach_time, 
                              float dist, float radius);

    // these are for move map projection only
    int avoid;
    int project;
    float decay;
    float radius;
    
public: 
     Actionmap();
     static void set_max_speed(float new_speed) { CUR_MAX_SPEED = new_speed;};
     float time2weight(float reach_time);
     float weight2time(float weight);
     float linear_value (float,float,float,float,float);
     float ball_sspd_vd(float end_speed, float distance);
     float ball_sspd_td(float reach_time, float distance);
     float ball_espd_vd(float start_speed, float distance);
     float ball_reach_time(float, float);
     float ball_spd2imp(float speed);
    
     void filter_times_longer_than(float time_longer_than);
     void filter_speeds_faster_than(float curr_max_speed)
          { amap.filter_speeds_faster_than(curr_max_speed);};
     void invert(){ amap.invert();}; 
     void mav(Actionmap& avoidmap, float accuracy_time);
     void use_mask(Actionmap& mask){amap.use_mask(mask.amap);};
     void use_invmask(Actionmap& mask){amap.use_invmask(mask.amap);};

     int find_best_action(Vector &best){ 
         return amap.find_best_action(best,CUR_MAX_SPEED);
     };
     float get_best_wgt(){ return amap.get_best_wgt();};
     int cpp_send(char* title){return amap.cpp_send(title);};
     int cpp_send(char* title, char* eps_file)
          {return amap.cpp_send(title, eps_file);};
     
     Maptype get_type(){return amap.get_type();};
     friend void operator+=(Actionmap& amapA, const Actionmap& amapB){amapA.amap += amapB.amap;};
     friend void operator-=(Actionmap& amapA, const Actionmap& amapB){amapA.amap -= amapB.amap;};
     friend void operator<<(Actionmap& amapA, const Actionmap& amapB){amapA.amap << amapB.amap;};
     void save_map(char* mapname){ amap.save_map(mapname);};

     float get_nrows()     {return amap.get_nrows();};
     float get_ncols()     {return amap.get_ncols();};
     float row2dir(int row) {return amap.row2dir(row); };
     float col2mgn(int col) {return amap.col2mgn(col); };
     float get_wgt(int row, int col) {return amap.get_wgt(row,col); }
private:
     float min_reach_t;
     float max_reach_t;
};
#endif
