// -*- mode:C++ -*-
// File:   Kickto_Goal_map.cc
// Author: Antti Tikanmki
// Contact:sunday@ee.oulu.fi
// Last c.:3. Jun 1998
#include <Am/Kickto_Goal_map.h>
Kickto_Goal_map::Kickto_Goal_map(void)
{
     obj_dir= obj_dist= ball_dir= ball_dist=0.0;
     amap.reset();
     amap.set_type(Kick);
     GOAL_WIDTH = Cons2::goal_width();
}

void Kickto_Goal_map::calculate_map(Vector ball_loc, Vector ball_vel,
                                    Vector goal_loc, Vector left_pole,
                                    Vector right_pole)
{
     reset();
     if(!kicks_calculated) calculate_possible_kicks(ball_loc, ball_vel);
     right_pole = right_pole - ball_loc;
     left_pole = left_pole - ball_loc;
     goal_loc = goal_loc - ball_loc;

     obj_dir = goal_loc.get_dir(); 
     ball_spd = ball_vel.get_len();
     
     float right_dir = right_pole.get_dir();
     float left_dir  = left_pole.get_dir();
     normalize_angle(right_dir);
     normalize_angle(left_dir); 
     
     float right_dist = right_pole.get_len();
     // Check the table over case
     int over = 0;
     if(left_dir<right_dir) over = 1;
     Vector goal;
     goal = left_pole-right_pole;
     float goal_angle = M_PI - goal.angle_from(right_pole);
     
     // Find the match kicks
     float wgt, cur_dist, dir_wgt;
     if(over)
          for(int row = 0;row < NRows; row++) {
               for(int col = NCols - 3;col < NCols; col++) {
                    if(kicks[row][col].dir < left_dir || kicks[row][col].dir > right_dir){
                         //if trying to slow the ball
                         if(kicks[row][col].vel < ball_spd) continue;
                         if(fabs(ball_vel.angle_from(goal_loc))<=0.25*M_PI &&
                            fabs(calc_angle_difference( amap.row2dir(row), obj_dir))>=0.25*M_PI)
                              continue;
                         cur_dist = right_dist *
                              (sin(goal_angle)/ sin(M_PI - angle_diff(kicks[row][col].dir, right_dir) - goal_angle));
                         dir_wgt = linear_value(fabs(angle_diff(kicks[row][col].dir, obj_dir)),0.0,0.5*M_PI, 1.0, 0.1 );
                         wgt = time2weight(ball_reach_time(kicks[row][col].vel ,cur_dist));
                         wgt *= dir_wgt;
                         amap.add_point(row, col, wgt,0);
                    }
               }
          }
     else  
          for(int row = 0;row < NRows; row++) {
               for(int col = NCols - 3;col < NCols; col++) {
                    if(kicks[row][col].dir < left_dir && kicks[row][col].dir > right_dir){
                         //if trying to slow the ball
                         if(kicks[row][col].vel < ball_spd) continue;
                         if(fabs(ball_vel.angle_from(goal_loc))<=0.25*M_PI &&
                            fabs(calc_angle_difference( amap.row2dir(row), obj_dir))>=0.25*M_PI)
                              continue;
                         cur_dist = right_dist *
                              (sin(goal_angle)/ sin(M_PI - angle_diff(kicks[row][col].dir, right_dir) - goal_angle));
                         dir_wgt = linear_value(fabs(angle_diff(kicks[row][col].dir, obj_dir)),0.0,0.5*M_PI, 1.0, 0.1 );
                         wgt = time2weight(ball_reach_time(kicks[row][col].vel ,cur_dist));
                         wgt *= dir_wgt;
                         amap.add_point(row, col, wgt,0);
                    }
               }
          }
}
float Kickto_Goal_map::GOAL_WIDTH;
