// -*- mode:C++ -*-
// File:   Kickto_Near_map.cc
// Author: Antti Tikanmki
// Contact:sunday@ee.oulu.fi
// Last c.:3. Jun 1998
#include <Am/Kickto_Near_map.h>

Kickto_Near_map::Kickto_Near_map(void)
{
     obj_dir= obj_dist= ball_dir= ball_dist=0.0;
     amap.reset();
     amap.set_type(Kick);
}
void Kickto_Near_map::calculate_map(Vector ball_loc, Vector ball_vel, Vector goal_loc)
{
     float HALF_SECTOR = 2.9;
     float SPEED_DIFF  = 0.3;
     float NEAR_WGT = Cons2::kick_near_wgt();
   
     reset();
     if(!kicks_calculated) calculate_possible_kicks(ball_loc, ball_vel);
     goal_loc = goal_loc - ball_loc;
    
     obj_dir = goal_loc.get_dir();

     // calculate direction limits
     float right_dir = goal_loc.get_dir() - HALF_SECTOR;
     normalize_angle(right_dir);
     float left_dir  = goal_loc.get_dir() + HALF_SECTOR;
     normalize_angle(left_dir);

     // Check the table over case
     int over = 0;
     if(left_dir<right_dir) over = 1;
     
     // Find the match kicks
     float dir_diff, wgt;
     float min_sspd = CUR_MAX_SPEED + Cons2::ball_near_speed() - SPEED_DIFF;
     float max_sspd = CUR_MAX_SPEED + Cons2::ball_near_speed() + SPEED_DIFF;
     
     for(int row = 0;row < NRows; row++) {
          for(int col = 0;col < NCols; col++) {
               if(over&&(kicks[row][col].dir < left_dir || kicks[row][col].dir > right_dir) ||
                  !over&&(kicks[row][col].dir < left_dir && kicks[row][col].dir > right_dir)) {
                    if(kicks[row][col].vel > min_sspd && kicks[row][col].vel < max_sspd){
                         dir_diff = fabs(calc_angle_difference(kicks[row][col].dir,obj_dir));
                         wgt = linear_value(dir_diff, 0.0, ANGLE_DIFF_LIMIT, NEAR_WGT, 0.0);
                         amap.add_point(row, col, wgt,0);
                    }
               }
          }    
     }
}
