// -*- mode:C++ -*-                                     
// File:   Kickto_Opp_map.h
// Author: Antti Tikanmki 
// Contact:sunday@ee.oulu.fi
// Last c.:2.Jun.1998 
#include <Am/Kickto_Opp_map.h>

Kickto_Opp_map::Kickto_Opp_map(void)
{
     obj_dir= obj_dist= ball_dir= ball_dist=0.0;
     amap.reset();
     amap.set_type(Kick);
     OPP_MAX_SPEED = Cons2::estimated_opp_spd();
     OPP_REACTION_TIME =Cons2::opp_reaction_time();
}

void Kickto_Opp_map::calculate_map(Vector ball_loc, Vector ball_vel,
                                   Vector player_loc, Vector player_vel)
{
     reset();
     if(!kicks_calculated) calculate_possible_kicks(ball_loc, ball_vel);
     player_loc = player_loc - ball_loc;

     obj_dist = player_loc.get_len();
     obj_dir  = player_loc.get_dir();

     float reaction_dst = find_max_vel_to_dir(obj_dir) * OPP_REACTION_TIME;

     // Calculate reach angle alfa
     float alfa;
     if(obj_dist>reaction_dst){
          float over_dist =min(15.0, obj_dist - reaction_dst);
         alfa = (float)asin((2.1 + over_dist)/obj_dist);
     }
     else if(obj_dist>=2.1) alfa = (float)asin(2.0/obj_dist);
     else                   alfa = M_PI * 0.5;
   
     // Calculate direction limits
     float right_dir = player_loc.get_dir() - alfa;
     normalize_angle(right_dir);
     float left_dir  = player_loc.get_dir() + alfa;
     normalize_angle(left_dir);
  
     // Check the table over case
     int over = 0;
     if(left_dir<right_dir) over = 1;

     // Find the match kicks & put wgt 
     float wgt;
     float min_sspd = 0.3;
     if(over)
          for(int row = 0;row < NRows; row++) {
               for(int col = 0;col < NCols; col++) {
                    if(kicks[row][col].dir < left_dir || kicks[row][col].dir > right_dir)
                         if(kicks[row][col].vel > min_sspd){
                              wgt = time2weight(ball_reach_time(kicks[row][col].vel,obj_dist));
                              amap.add_point(row,col,wgt,0);
                         }
               }
          }
     else  
          for(int row = 0;row < NRows; row++) {
               for(int col = 0;col < NCols; col++) {
                    if(kicks[row][col].dir < left_dir && kicks[row][col].dir > right_dir)
                         if(kicks[row][col].vel > min_sspd){
                              wgt = time2weight(ball_reach_time(kicks[row][col].vel,obj_dist));
                              amap.add_point(row,col,wgt,0);
                         }
               }
          }
}

float Kickto_Opp_map::OPP_MAX_SPEED;
float Kickto_Opp_map::OPP_REACTION_TIME;
