// -*- mode:C++ -*-                                     
// File:   Kickto_Teammate_map.cc
// Author: Antti Tikanmki
// Contact:sunday@ee.oulu.fi
// Last c.: 4.Jun.1998
#include <Am/Kickto_Teammate_map.h>
#include <math.h>
Kickto_Teammate_map::Kickto_Teammate_map(void)
{
     obj_dir=obj_dist=ball_dir=ball_dist=0.0;
     amap.reset();
     amap.set_type(Kick);
     PASS_END_SPEED = Cons2::pass_end_spd();    
}

void Kickto_Teammate_map::calculate_map(Vector ball_loc, Vector ball_vel,
                                   Vector player_loc, Vector player_vel)
{
     float TEAMMATE_REACTION_TIME = 2.0;
     float PASS_END_SPEED_RANGE   = 0.2;// Difference of ball's end speed at teammate

     reset();
     if(!kicks_calculated) calculate_possible_kicks(ball_loc, ball_vel);
     player_loc = player_loc - ball_loc;

     obj_dir  = player_loc.get_dir();
     obj_dist = player_loc.get_len();

     float reaction_dst = find_max_vel_to_dir(obj_dir) * TEAMMATE_REACTION_TIME;

     // Calculate reach angle alfa
     float alfa;
     if(obj_dist>reaction_dst){
          float over_dist =min(15.0, obj_dist - reaction_dst);
          alfa = (float)asin((2.0 + (over_dist))/obj_dist);
     }
     else if(obj_dist>=2.0) alfa = (float)asin(2.0/obj_dist);
     else                   alfa = M_PI * 0.5;
     
     // Calculate direction limits
     float right_dir = player_loc.get_dir() - alfa;
     normalize_angle(right_dir);
     float left_dir  = player_loc.get_dir() + alfa;
     normalize_angle(left_dir);
  
     // Check the table over case
     int over = 0;
     if(left_dir<right_dir) over = 1;

     // Find the match kicks & put wgt 
     float dir_diff, wgt, timewgt;
     float min_sspd = ball_sspd_vd(PASS_END_SPEED - PASS_END_SPEED_RANGE, obj_dist);
     float max_sspd = ball_sspd_vd(PASS_END_SPEED + PASS_END_SPEED_RANGE, obj_dist);

     if(over)
          for(int row = 0;row < NRows; row++) {
               for(int col = 0;col < NCols; col++) {
                    if(kicks[row][col].dir < left_dir || kicks[row][col].dir > right_dir)
                         if(kicks[row][col].vel >= min_sspd && kicks[row][col].vel <= max_sspd){
                               if(fabs(ball_vel.angle_from(player_loc))<=0.25*M_PI &&
                                 fabs(calc_angle_difference( amap.row2dir(row), obj_dir))>=0.25*M_PI)
                                   continue;
                               dir_diff = fabs(calc_angle_difference(kicks[row][col].dir,obj_dir));
                               timewgt = time2weight(ball_reach_time(kicks[row][col].vel,obj_dist));
                               wgt = linear_value(dir_diff, 0.0, ANGLE_DIFF_LIMIT, timewgt, 0.0);
                               amap.add_point(row,col,wgt,0);
                         }
               }
          }
     else for(int row = 0;row < NRows; row++) {
          for(int col = 0;col < NCols; col++) {
                    if(kicks[row][col].dir < left_dir && kicks[row][col].dir > right_dir)
                         if(kicks[row][col].vel >= min_sspd && kicks[row][col].vel <= max_sspd){
                              if(fabs(ball_vel.angle_from(player_loc))<=0.25*M_PI &&
                                 fabs(calc_angle_difference( amap.row2dir(row), obj_dir))>=0.25*M_PI)
                                   continue;
                              dir_diff = fabs(calc_angle_difference(kicks[row][col].dir,obj_dir));
                              timewgt = time2weight(ball_reach_time(kicks[row][col].vel,obj_dist));
                              wgt = linear_value(dir_diff, 0.0, ANGLE_DIFF_LIMIT, timewgt, 0.0);
                              amap.add_point(row,col,wgt,0);
                         }
          }
     }
     return;
}

float Kickto_Teammate_map::PASS_END_SPEED;
