// File     :  Ball_state.cc
// Author   :  Jussi Pajala
// Date     :  30.7./1997

// 30.7.1997  V.0.5     The first version
// 1.8.1997   V.0.6     One (big) bug fixed
// 20.2.1998  v.1.5     A big change

#include <stdlib.h>
#include <iostream.h>
#include <assert.h>
#include <Ma/Ball_state.h>
#include "Cons2.h"
//#define BALL_STATE_DEBUG
#ifdef BALL_STATE_DEBUG
#include <fstream.h>
ofstream me_fout;
#endif
  
const MAX_TIME_LEAP = 8;
int printtaa = 0;

// INPUTS
// int adv       : tell's how many time instants execution delay is
// float vel_red : velocity is multiplied by this factor at each simulation
//                 cycle
Ball_state::Ball_state(int system_delay, float vel_red)
{
    
    delay = system_delay;
    NSTATES = delay + 1 + MAX_TIME_LEAP;
    state = new BState[NSTATES];
    
    decay = vel_red;
    curr_time = 0;
#ifdef BALL_FILTER
    xfilter = new Ab_ball_filter(0.9);
    yfilter = new Ab_ball_filter(0.9);
#endif

    reset();
    
}

Ball_state::~Ball_state()
{
    delete state;
}

void Ball_state::reset()
{
    for( int i = 0; i < NSTATES; i++){
        state[i].time = 0;
        state[i].vel.reset();
        state[i].vel_set = 0;
        state[i].x = 0.0;
        state[i].y = 0.0;
    }
}


// updates the array so that time instances are in sequence.
// also takes care of movements and coordinate updates
void Ball_state::fill_array()
{
    int i;
    
    for(i = 1; i < NSTATES; i++){
        if( state[i].time <= state[i-1].time ){
            // if execution reaches this point, we have found an empty
            // array slot. If this is the case, time should be zero.
            // If time is something else, something wierd has happened...
            if(state[i].time){ 
                //cerr << "i: " << i << " t: " << state[i].time 
                // << endl;
                //print_all_states();
            }
            
            //assert(state[i].time == 0);
            state[i].time = state[i-1].time + 1;
            state[i].vel_set = 0;
        }
        //if speed is not set for this slot, 
        // approximate it with help of previous speed
        if( !state[i].vel_set )
            state[i].vel = decay * state[i-1].vel;
                
        // coordinates can be easily approximated with previous coords and
        // previous speed.
        state[i].x = state[i-1].x 
            + state[i-1].vel.get_x();
        state[i].y = state[i-1].y 
            + state[i-1].vel.get_y();
        if(state[i].x < MIN_X || state[i].x > MAX_X || state[i].y < MIN_Y 
           || state[i].y > MAX_Y){
            state[i].vel.reset();
            state[i].vel_set = 1;
        }
    }
}


void Ball_state::set_curr_time(int time)
{
    int i;
    int old_time = curr_time;
    curr_time = time;
    //cerr << "set_curr_time" << endl;


    if(state[delay+1].vel_set){
        printtaa = 1;
        //cerr << "NOW VEL IS SET!"<< endl;
        //print_all_states();
    }

    for(i = 0; i < NSTATES;i++){
        if(state[i].time == time){
            if(i > 0 && state[i].vel_set)
                if((state[i].rob_x-state[i].x)*(state[i].rob_x-state[i].x)+
                   (state[i].rob_y-state[i].y)*(state[i].rob_y-state[i].y)
                   > Cons2::kick_max_dist())
                    state[i].vel_set = 0;
            i = NSTATES;
        }
    }
    
        
    fill_array();
         
    int gap = curr_time - old_time;
    if(gap < 0){
//        cerr << __FILE__ << " time gap negative!" << endl;
        for(i = 0; i < NSTATES; i++) if(state[i].time != 0) 
            state[i].time += gap;
        gap = 0;
    } else {
        
    
        if(!gap){
            for( i= 0; i < NSTATES; i++)
                if(state[i].time >= curr_time) state[i].time = state[i].time -1;
            gap = 1;
        }
        
//   if(gap){ // return; // there is nothing to do if time gap is zero!
        if(gap>NSTATES-1){
            // this should be a very special case!
#ifndef NOPRINTS
            cerr << "Initializing Ball_state! " << endl;
#endif
            state[0].time = time;
            fill_array();
        }
        else{
            // move current time to first place of the array
            for(i = 0; i+gap < NSTATES; i++)
                state[i] = state[i+gap];
            // jp 5.3.1998
            state[0].time = curr_time;
                      
            // if gap + delay >= NSTATES, it means that MAX_TIME_LEAP is too 
            // small. This means that server time is at least MAX_TIME_LEAP
            // ahead of the Ball time.
            assert( gap + delay < NSTATES );
            
            for(i = NSTATES-gap; i < NSTATES; i++){
                state[i].time = 0;
                state[i].vel.reset();
                state[i].x = 0.0;
                state[i].y = 0.0;
            }
            assert( state[0].time == curr_time );
        }
   }
   state[delay + 1].vel_set = 0;
   if(printtaa){
       //cerr << "LETS SEE WHAT IT'S NOW!" << endl;
       //print_all_states();
   }
}

void Ball_state::set_rob_pos(Vector rob_pos,int time)
{
    int i;
    for(i = 0; i < NSTATES;i++)
        if(state[i].time == time){
            state[i].rob_x = rob_pos.get_x();
            state[i].rob_y = rob_pos.get_y();
        }
}



void Ball_state::set_curr_g_pos(Vector g_pos)
{
//    cerr << "set curr pos" << endl;
    int i;
    float gap_x, gap_y;

#ifdef BALL_FILTER
    xfilter->smooth(state[0].time, g_pos.get_x(), filter_x, filter_vx);
    yfilter->smooth(state[0].time, g_pos.get_y(), filter_y, filter_vy);
    gap_x = filter_x - state[0].x;
    gap_y = filter_y - state[0].y;
#else
    gap_x = g_pos.get_x() - state[0].x;
    gap_y = g_pos.get_y() - state[0].y;
#endif
    
//    cerr << "gap_x = " << gap_x << " gap_y = " << gap_y << endl;
    
    for(i = 0; i < NSTATES; i++){
        state[i].x += gap_x;
        state[i].y += gap_y;
    }
}

void Ball_state::set_curr_g_vel(Vector vel)
{
#ifndef BALL_FILTER
//     cerr << "set curr speed" << endl;
    Vector gap = vel - state[0].vel;
#else 
    Vector temp;
    temp.set_xy(filter_vx, filter_vy);
    Vector gap = temp - state[0].vel;
#endif
    int i;
    for(i = 0; i < NSTATES; i++){
        if(!state[i].vel_set) state[i].vel = state[i].vel + gap;
        gap = decay * gap;
    }
}

void Ball_state::set_target_vel(Vector vel)
{
//    cerr << "target_speed" << endl;
    
    fill_array();

    state[delay+1].vel = vel;
    state[delay+1].vel_set = 1;
    state[delay+1].time = state[0].time + delay + 1;
    
    //cerr << "VELOCITY SET" << endl;
//    print_all_states();
    
}

void Ball_state::get_exec_params(Vector &v, Vector &g_pos)
{
//     cerr << "get exec params" << endl;
    if(printtaa){
        printtaa = 0;
        //cerr << "AND NOW GETTING EXEC DATA" << endl;
        //print_all_states();
    }
    
    v = state[delay].vel;
    g_pos.set_xy(state[delay].x, state[delay].y);
//    cerr << "GETTING EXEC_PARAMS FROM INDEX " << delay << endl;
//    print_all_states();
}

void Ball_state::get_curr_params(Vector &v, Vector &g_pos)
{
//     cerr << "get curr params" << endl;
     
    v = state[0].vel;
    g_pos.set_xy(state[0].x, state[0].y);
}

void Ball_state::print_all_states()
{
#ifndef NOPRINTS

     int i;
     for(i = 0; i < NSTATES; i++)
     cerr << "i " << i << " t " << state[i].time 
          << " spd " << state[i].vel.get_len()
          << " spd dir " << state[i].vel.get_dir()
          << " vel_set " << state[i].vel_set
          << " x " << state[i].x
          << " y " << state[i].y << endl;
#endif
}








