// File:    Ma_border.h
// Author:  Jussi Pajala 3.6.1997, jussip@tks.oulu.fi


#include "decls.h"
//#include "Signal.h"
#include <Ma/Ma_border.h>


Ma_border::Ma_border()
{
     my_id = MA_BORDER_ID;
     inhibited = NO;
     
     if(Cons2::debug_file_ma_border()){
          char filename[100];
          sprintf(filename, "data/%d_border.txt", global_me_unum);
          trc_init((int)my_id,filename, "Border marker\ntime jotain debuggia tnne!!\n");
          trc_init_event((int)my_id, 1, "%d %f \n", 2);
     }
     TOP_LEFT.set_xy(MIN_X,MAX_Y);
     TOP_RIGHT.set_xy(MAX_X,MAX_Y);
     RIGHT_GOAL_TOP.set_xy(MAX_X,GOAL_WIDTH/2);
     RIGHT_GOAL_BOTTOM.set_xy(MAX_X,-GOAL_WIDTH/2);
     BOTTOM_RIGHT.set_xy(MAX_X,MIN_Y);
     BOTTOM_LEFT.set_xy(MIN_X,MIN_Y);
     LEFT_GOAL_BOTTOM.set_xy(MIN_X,-GOAL_WIDTH/2);
     LEFT_GOAL_TOP.set_xy(MIN_X,GOAL_WIDTH/2);

     if(Cons2::debug_level_startup()) cerr << "Ma_border: CONSTRUCTOR executed" << endl;
}

Ma_border::~Ma_border()
{
     if(Cons2::debug_file_ma_border()){
          trc_save((int)my_id);
          trc_clear((int)my_id);
     }
     if(Cons2::debug_level_exit()) cerr << "Ma_border: DESTRUCTOR executed" << endl;
}

void Ma_border::Init(EmptySamba *sambapointer)
{
     samba = sambapointer;
     sensor = samba->get_pointer( SENSOR_ID );
     ma_me = samba->get_pointer( MA_ME_ID );
}

int Ma_border::transform()
{
     Line_info T,B,Lt,Lb,Rt,Rb; // Line information of all the lines:
                                // top, bottom, lefttop, leftbottom, righttop, rightbottom

     float x,y;
     float Tx,Bx,Rty,Rby,Lty,Lby;
     
     
     me_msig_in->signal.read(me_g_loc, me_g_vel, me_g_d_loc, me_heading,
                             me_d_heading, me_exec_loc, me_exec_vel,
                             me_exec_heading, me_stamina, me_unum,
                             me_g_cer, me_time, me_id);

     if(global_marker_advance){
          x = me_exec_loc.get_x();
          y = me_exec_loc.get_y();
     } else {
          x = me_g_loc.get_x();
          y = me_g_loc.get_y();
     }
               
     // rlen ja llen on viivan pituus lhimmn pisteen oikealla ja
     // vasemmalla puolella robottiin nhden, kun se katsoo suoraan
     // ko pisteeseen
              
     if(x < MIN_X) {
          Tx = MIN_X;
          Bx = MIN_X;
          T.llen = B.rlen = 0.0;
          T.rlen = B.llen = 2*MAX_X;
     } else if(x > MAX_X){
          Tx = MAX_X;
          Bx = MAX_X;
          T.llen = B.rlen = 2*MAX_X;
          T.rlen = B.llen = 0.0;

     } else {
          Tx = x;
          Bx = x;
          T.llen = B.rlen = x - MIN_X;
          T.rlen = B.llen = MAX_X - x;
     }

     if(y < MIN_Y){
          Rby = MIN_Y;
          Lby = MIN_Y;
          Rb.rlen = Lb.llen = 0.0;
          Rb.llen = Lb.rlen = -GOAL_WIDTH/2 - MIN_Y;
     } else if(y < -GOAL_WIDTH/2) {
          Rby = y;
          Lby = y;
          Rb.rlen = Lb.llen = y - MIN_Y;
          Rb.llen = Lb.rlen = -GOAL_WIDTH/2 - y;
     } else {
          Rby =  -GOAL_WIDTH/2;
          Lby =  -GOAL_WIDTH/2;
          Rb.rlen = Lb.llen = -GOAL_WIDTH/2 - MIN_Y;
          Rb.llen = Lb.rlen = 0.0;
     }

     if(y > MAX_Y){
          Rty = MAX_Y;
          Lty = MAX_Y;
          Rt.rlen = Lt.llen = MAX_Y - GOAL_WIDTH/2;
          Rt.llen = Lt.rlen = 0.0;
     } else if(y > GOAL_WIDTH/2) {
          Rty = y;
          Lty = y;
          Rt.rlen = Lt.llen = y - GOAL_WIDTH/2;
          Rt.llen = Lt.rlen = MAX_Y - y;
     } else {
          Rty =  GOAL_WIDTH/2;
          Lty =  GOAL_WIDTH/2;
          Rt.rlen = Lt.llen = 0.0;
          Rt.llen = Lt.rlen = MAX_Y - GOAL_WIDTH/2;
     }

     // calculate angles to all the lines
     if(global_marker_advance){
          T.angle = me_exec_heading;
          Rb.angle = Rt.angle = me_exec_heading - ( 0.5 * M_PI );
          B.angle = me_exec_heading + M_PI;
          Lb.angle = Lt.angle = me_exec_heading + ( 0.5 * M_PI );  // + 1.5*M_PI;
     } else {
          T.angle = me_heading;
          Rb.angle = Rt.angle = me_heading - ( 0.5 * M_PI );
          B.angle = me_heading + M_PI;
          Lb.angle = Lt.angle = me_heading + ( 0.5 * M_PI );  // + 1.5*M_PI;
     }
     
     float temp;
     // if we are on wrong side of the borders, we must swap left to right and right to left
     if(x < MIN_X){ // swap llen and rlen of line L
          temp = Lt.rlen;
          Lt.rlen = Lt.llen;
          Lt.llen = temp;
          Lt.angle -= M_PI;

          temp = Lb.rlen;
          Lb.rlen = Lb.llen;
          Lb.llen = temp;
          Lb.angle -= M_PI;
     }
     if(x > MAX_X){
          temp = Rt.rlen;
          Rt.rlen = Rt.llen;
          Rt.llen = temp;
          Rt.angle -= M_PI;

          temp = Rb.rlen;
          Rb.rlen = Rb.llen;
          Rb.llen = temp;
          Rb.angle -= M_PI;
     }
     if(y < MIN_Y){
          temp = B.rlen;
          B.rlen = B.llen;
          B.llen = temp;
          B.angle -= M_PI;
     }
     if(y > MAX_Y){ 
          temp = T.rlen;
          T.rlen = T.llen;
          T.llen = temp;
          T.angle -= M_PI;
     }

     normalize_angle( T.angle );
     normalize_angle( Rt.angle );
     normalize_angle( Rb.angle );
     normalize_angle( B.angle );
     normalize_angle( Lt.angle );
     normalize_angle( Lb.angle );
     
     
     // tss on sstetty vhn prosessoriaikaa.
     // Vektorilaskennalla allaolevat laskut vektorille T:
     // T.set_xy(Tx,MAX_Y);
     // T = T - me_g_loc;
     // T.turn(-me_heading);
     // tss vektoreiden vhennyslaskut tehdn vhentmll x ja y komponentit
     // toisistaan ja vasta sitten muodostetaan vektori

     // compute difference between vector to nearest point of
     // a line and vector to global position of the robot

     T.loc.set_xy(Tx - x, MAX_Y - y);    // top
     B.loc.set_xy(Bx - x, MIN_Y - y);    // bottom

     Rt.loc.set_xy(MAX_X - x,Rty - y );  // right top
     Rb.loc.set_xy(MAX_X - x,Rby - y );  // right bottom

     Lt.loc.set_xy(MIN_X - x,Lty - y );  // left top
     Lb.loc.set_xy(MIN_X - x,Lby - y );  // left bottom
     
     // turn vector to local coordinates
     if( global_marker_advance ){
          T.loc.turn(-me_exec_heading);
          B.loc.turn(-me_exec_heading);
          
          Rt.loc.turn(-me_exec_heading);
          Rb.loc.turn(-me_exec_heading);
          
          Lt.loc.turn(-me_exec_heading);
          Lb.loc.turn(-me_exec_heading);
     } else {
          T.loc.turn(-me_heading);
          B.loc.turn(-me_heading);
          
          Rt.loc.turn(-me_heading);
          Rb.loc.turn(-me_heading);
          
          Lt.loc.turn(-me_heading);
          Lb.loc.turn(-me_heading);
     }
     
     T.start_g_loc = TOP_LEFT;
     T.stop_g_loc = TOP_RIGHT;

     Rt.start_g_loc = TOP_RIGHT;
     Rt.stop_g_loc = RIGHT_GOAL_TOP;

     Rb.start_g_loc = RIGHT_GOAL_BOTTOM;
     Rb.stop_g_loc = BOTTOM_RIGHT;

     B.start_g_loc = BOTTOM_RIGHT;
     B.stop_g_loc = BOTTOM_LEFT;

     Lb.start_g_loc = BOTTOM_LEFT;
     Lb.stop_g_loc = LEFT_GOAL_BOTTOM;

     Lt.start_g_loc = LEFT_GOAL_TOP;
     Lt.stop_g_loc = TOP_LEFT;

     T.cer = B.cer = Rt.cer = Rb.cer = Lt.cer = Lb.cer = me_g_cer;

     lines.T = T;
     lines.B = B;
     lines.Rb = Rb;
     lines.Rt = Rt;
     Lines.Lb = Lb;
     lines.Lt = Lt;
     lines.C = unknown!!!!
     
// write to debugfile     
     if(Const2::debug_file_ma_border())
          trc((int)my_id, 1, curr_time, T.llen, T.rlen);
                  
     
     if(Const2::debug_level() > 4) cerr << "  Ma_border: ending transformation... " << endl;
     
     return 0;
}

