// file    : Ma_defloc.cc   

#include "decls.h"
//#include "Soccer.h"
#include <Ma/Ma_defloc.h>

const float FILTER_MIN_DIST = 10.0; // minimum distance where filter can be used


Ma_defloc::Ma_defloc()
{
     my_id = MA_DEFLOC_ID;   
     inhibited = NO;
     first_time = 1;

     start_g_loc.set_xy(0.5*(Cons2::att_x() + Cons2::def_x()),
                        0.5*(Cons2::att_y() + Cons2::def_y()));
          
     if(Cons2::debug_level_startup()) cerr << "Ma_defloc: CONSTRUCTOR executed" << endl;
}

Ma_defloc::~Ma_defloc()
{
     if(Cons2::debug_level_exit()) cerr << "Ma_defloc: DESTRUCTOR executed" << endl;
}

void Ma_defloc::Init(EmptySamba *sambapointer)
{
    // store pointer of the control software main class.
     samba = sambapointer;

     ma_me = (Ma_me*)samba->get_pointer( MA_ME_ID );
     ma_ball = (Ma_ball*)samba->get_pointer( MA_BALL_ID );
     ma_goal = (Ma_goal*)samba->get_pointer( MA_GOAL_ID );

     vs_referee = (Vs_referee*)samba->get_pointer(VS_REFEREE_ID);

}

void Ma_defloc::calculate_goal_keeper_defloc()
{
     ma_ball->read(ball);
     ma_goal->read_own_goal(own_goal);
          
     // Values for virtual circle center behind the goal:
     // Add configuration possibility here!!:
     float circ_x = -65.25;        // 12.75 meters behind the goal=(-52.5,0.0)
     float circ_y = 0.0;
     // jpr aug18: I moved the defensive location towards the middle, the same here
     // old: float circ_rad = 14.55;
     float circ_rad = 15.55;
     
     Vector goal2up(0.0, 7.01);    // from goal center to upper pole
     Vector goal2bot(0.0,-7.01);   // from goal center to lower pole

     float ball_x = ball.g_loc.get_x();
     float ball_y = ball.g_loc.get_y();
     
     Vector ball2up(0.0, 0.0);     // from ball to upper pole
     Vector ball2bot(0.0, 0.0);    // from ball to lower pole
     float ball2up_dir=0.0;
     float ball2bot_dir=0.0;
     float i1_x = 0.0;
     float i1_y = 0.0;
     float i2_x = 0.0;
     float i2_y = 0.0;
     float line_dir = 0.0;
     int result = 0;

     // Calculation of line from ball to goal.
     // The line separates the sector from ball to goal to
     // two separate parts with same angles.
     ball2up      = (own_goal.g_loc.center + goal2up)  - ball.g_loc;
     ball2bot     = (own_goal.g_loc.center + goal2bot) - ball.g_loc;
     ball2up_dir  = ball2up.get_dir();
     ball2bot_dir = ball2bot.get_dir();

     // Alfa = angle of sector from ball to goal
     float alfa = angle_diff(ball2bot_dir, ball2up_dir);
     line_dir = calc_angle_sum(ball2up_dir, alfa/2.0);
     
     result = line_circle_intersection
          (ball_x, ball_y, line_dir, circ_x, circ_y, circ_rad,
           i1_x, i1_y, i2_x, i2_y);

     // Function returns global location of goalkeeper's defloc:
     if(result==2) defloc.g_loc.set_xy(i1_x,i1_y);
     
     // If calculation is not valid.
     // --> The defloc is set 2.0 meters before goal center.
     else defloc.g_loc.set_xy((own_goal.g_loc.center.get_x())+2.0 , 0.0);
     return;
     
} // End of calculate_goal_keeper_defloc 



void Ma_defloc::read(Defloc_marker &to)
{
     check_executed();
     to = defloc;
}

void Ma_defloc::invalidate_all()
{
    defloc.valid = 0;
}

     
int Ma_defloc::transform()
{
     ma_me->read_exec(me_exec);
     ma_ball->read(ball);
     
     vs_referee->read(referee);
     
     defence_wgt = 1.0 - (ball.g_loc.get_x() - MIN_X)/(2*MAX_X);
     
     // if this is the first time, do some calculations
     if(first_time){
          first_time = 0;
          defence_g_loc.set_xy(Cons2::def_x(),Cons2::def_y());
          attack_g_loc.set_xy(Cons2::att_x(),Cons2::att_y());
          defence_to_attack = attack_g_loc - defence_g_loc;
     }

     if( !referee.time_running  ){
         defloc.g_loc = start_g_loc; // was defloc_g_loc!!
     }
          
     // if robot outside the field, calculate defloc to 
     // ten meters inside the field
     else if( me_exec.g_loc.get_x() < MIN_X
              || me_exec.g_loc.get_x() > MAX_X
              || me_exec.g_loc.get_y() < MIN_Y
              || me_exec.g_loc.get_y() > MAX_Y ){
          float x = me_exec.g_loc.get_x(),
                y = me_exec.g_loc.get_y();
          if(y < MIN_Y) y = MIN_Y + 10.0;
          else if( y > MAX_Y ) y = MAX_Y - 10.0;
          if( x  < MIN_X) x = MIN_X + 10.0;
          else if( x > MAX_X ) x = MAX_X - 10.0;
          
          defloc.g_loc.set_xy(x, y);
     }
     // else if goalie, compute special defloc to it
     else if(Cons2::me_unum() == GOAL_KEEPER_UNUM){
          calculate_goal_keeper_defloc();
          } 
     // else for all other players calculate defloc for all different 
     // situations
     else {
              
         defloc.g_loc = attack_g_loc - defence_wgt * defence_to_attack;
         
         // if(( ( curr_time < 1000) 
         //     ||( curr_time > 3000 && curr_time < 3500))
         //   && referee.opp_goals == 0 && attack_g_loc.get_x() > 0){    
         //cerr << "setting defloc for testing here in " 
         //     << __FILE__ << " " << __LINE__ << endl;
         //defloc.g_loc = defence_g_loc;
         //}
         
         if((referee.game_state == team_kick_in ||
             referee.game_state == team_corner_kick ||
             referee.game_state == team_free_kick ) &&
            ball.g_loc.get_x()> 10.0){
             defloc.g_loc = attack_g_loc;
         }
         
         if((referee.game_state == opp_kick_in ||
             referee.game_state == opp_corner_kick ||
             referee.game_state == opp_free_kick   ) &&
            ball.g_loc.get_x()< 0.0){
             defloc.g_loc = defence_g_loc;
         }
         if( referee.game_state == opp_goal_kick ) defloc.g_loc = attack_g_loc;
         if( referee.game_state == team_goal_kick ) defloc.g_loc = defence_g_loc;
     
         // if we are not allowed to go to opponents side, check that defloc is
         // at least one meter on our side 
         if( (!referee.time_running || referee.game_state == opp_kick_off 
              || referee.game_state == team_kick_off) 
             && defloc.g_loc.get_x() > -1.0 )
             defloc.g_loc.set_xy( -1.0, defloc.g_loc.get_y());
     }
     
     // compute local situation
     defloc.loc = defloc.g_loc - me_exec.g_loc;
     defloc.loc.turn(-me_exec.heading);
     
     // if defloc further than 20 meters calculate 'virtual defloc' to
     // same direction than actual defloc but only 20 meters away
     if(defloc.loc.get_len() > 20.0) 
         defloc.loc.set_dir_len(defloc.loc.get_dir(),20.0);
     
     defloc.valid = 1;
     
     return 0;
}
