// file    : Ma_offside.cc
#include "decls.h"
//#include "Soccer.h"
#include <Ma/Ma_offside.h>

Ma_offside::Ma_offside()
{
     my_id = MA_OFFSIDE_ID;   
     inhibited = NO;
     //first_time = 1;
     if(Cons2::debug_level_startup()) cerr << "Ma_offside: CONSTRUCTOR executed" << endl;
}

Ma_offside::~Ma_offside()
{
     if(Cons2::debug_level_exit()) cerr << "Ma_offside: DESTRUCTOR executed" << endl;
}

void Ma_offside::Init(EmptySamba *sambapointer)
{
    // store pointer of the control software main class.
     samba = sambapointer;
     ma_me = (Ma_me*)samba->get_pointer( MA_ME_ID );
     ma_ball = (Ma_ball*)samba->get_pointer( MA_BALL_ID );
     ma_opponent = (Ma_opponent*)samba->get_pointer( MA_OPPONENT_ID );
     vs_referee = (Vs_referee*)samba->get_pointer( VS_REFEREE_ID );
}

void Ma_offside::read(Offside_marker &to)
{
     check_executed();
     to = offside;
}

void Ma_offside::invalidate_all()
{
    offside.valid = 0;
}
     
int Ma_offside::transform()
{
     ma_me->read_exec(me_exec);
     ma_ball->read(ball);
     ma_opponent->read(opponents);
     vs_referee->read(referee);

     // can't be offside position if on own side!
     if(me_exec.g_loc.get_x() < 0.0) return 0;
     
     
     float opp_x = 0.0;
     float last_opp_x = 0.0;
     float second_last_opp_x = 0.0;
     float offside_x = 0.0;
     
     for(int i=0;i<11;i++){
          if(opponents.player[i].cer < 0.1) continue;
          opp_x = opponents.player[i].g_loc.get_x();
          if(opp_x >= last_opp_x){
               second_last_opp_x = last_opp_x;
               last_opp_x = opp_x;
          }
          else if(opp_x > second_last_opp_x)
               second_last_opp_x = opp_x;
     }
     offside_x = second_last_opp_x;
     
     // offside location can't be behind the ball
     if(offside_x < ball.g_loc.get_x())
         offside_x = ball.g_loc.get_x();
     // offside location can't be on own side
     if(offside_x < 0.0) offside_x = 0.0;
     
     if( referee.game_state == opp_offside ) 
         offside_x = ball.g_loc.get_x() -5.0;
     
     // offside marker not valid if player not in offside location
     if(me_exec.g_loc.get_x() < offside_x) return 0;
     
     offside.g_loc.set_xy(offside_x - 2.0, me_exec.g_loc.get_y());
     
     offside.loc = g_loc2loc(offside.g_loc, me_exec.g_loc, me_exec.heading);
     // if offside location further away than 20 meters, move it nearer in 
     // local coordinates
     if(offside.loc.get_len() > 20.0) 
         offside.loc.set_dir_len(offside.loc.get_dir(), 20.0);
     /*cerr << "Offside g_loc: ";
     offside.g_loc.err_print_xy();
     cerr << " dir len: ";
     offside.g_loc.err_print_dir_len();
     cerr << endl;
     */
     
     offside.valid = 1;
     return 0;
}
