/* file    : Ma_specloc.loc.cc
 * 
 */

#include "decls.h"
#include <Ma/Ma_special_loc.h>

Ma_special_loc::Ma_special_loc ()
{
     my_id = MA_SPECIAL_LOC_ID;
     inhibited = NO;
     
     if(Cons2::debug_level_startup()) cerr << "Ma_specloc.loc: CONSTRUCTOR executed" << endl;
}


Ma_special_loc::~Ma_special_loc ()
{
    // DEBUG
    if(Cons2::debug_level_exit()) 
        cerr << "Ma_specloc.loc: DESTRUCTOR executed" << endl;
}

void Ma_special_loc::Init(EmptySamba *sambapointer)
{
    samba = sambapointer;
    ma_me =   (Ma_me*)  samba->get_pointer( MA_ME_ID );
    ma_ball = (Ma_ball*)samba->get_pointer( MA_BALL_ID );
    ma_goal = (Ma_goal*)samba->get_pointer( MA_GOAL_ID );
}

void Ma_special_loc::read(Special_loc &to)
{
    check_executed();
    to = specloc;
}

void Ma_special_loc::invalidate_all()
{
    specloc.valid = 0;
}

int Ma_special_loc::transform ()
{
    static Vector goal2ball;
    static Vector ball2specloc;
    static Vector me2ball;
    
     ma_me->read_exec( me );
     ma_ball->read( ball );
     ma_goal->read_opp_goal( opp_goal );
          
     // return if certainties are not high enough
     if(ball.cer < 0.1 || opp_goal.cer < 0.1 || me.g_cer < 0.1){
         return 0;
     }
          
     // calculate vector from opponent goal to ball
     goal2ball = ball.g_loc - opp_goal.g_loc.center;
     
     // calculate vector ball2specloc, use goal2ball as a base vector
     ball2specloc = goal2ball;
     // decrease vector length to 1.5 meters
     ball2specloc.lengthen(-ball2specloc.get_len() + 1.0);
     if(ball.g_loc.get_y() > 0) ball2specloc.turn( M_PI*0.5 );
     else ball2specloc.turn( -M_PI*0.5);
     
/*     me2ball = ball.g_loc - me.g_loc;
  
     float angle = me2ball.angle_to(ball2specloc);
     
     if( fabs( angle ) < M_PI/2.0 && ball.loc.get_len() > 3.0){
         // if we come here special location is behind the ball and
         // we are not close to ball,
         // turn vector ball2specloc so that it is not behind
         if( angle < 0) ball2specloc.turn( -M_PI/2.0 - angle );
         else ball2specloc.turn( M_PI/2.0 - angle );
         ball2specloc.lengthen(1.0);
         
     }
*/   
     
     // location of the special location is one meter from the ball
     // to direction opposite to goal direction
     specloc.g_loc = ball.g_loc + ball2specloc;

     specloc.loc = specloc.g_loc - me.g_loc;
     specloc.loc.turn( -me.heading );
          
     /*cerr << "LOC: ";
       specloc.loc.err_print_dir_len();
     cerr << endl << "GLOC";
     specloc.g_loc.err_print_xy();*/
     
     specloc.cer = (ball.cer + opp_goal.cer + me.g_cer) / 3.0;
     specloc.valid = 1;
     
     return 0;
}



