// File:   Module.cc
// Author: Jussi Pajala 3.3.1997,  jussip@tks.oulu.fi

#include "decls.h"
#include "Module.h"
#include "Samba.h"

//
// Module Constructor
//

Module::Module() {
     my_id = DEFAULT_ID;
     inhibited = NO;
     executed = NO;
}
//
// Destructor
//
Module::~Module()
{
}

void Module::Init(EmptySamba *sambapointer)
{
    // this function should never be executed if the module must
    // be connected to some other modules. Module pointers should
    // be requested from Samba module here in Init function
    samba = sambapointer;
    if(Const::debug_level_startup() || Const::debug_level()){
        if(my_id == SENSOR_ID) return; // ok, sensor need no init function
        cerr << "WARNING: virtual Init function of the "
             << "superclass Module executed! ("
             << Const::modulename(my_id) << ")" << endl;
    }
    
    return;
}

void Module::execute()
{
    if(executed) {
        if(Const::debug_level()>7) 
            cerr << "Not executing module " << Const::modulename(my_id) 
                 << " because already executed..." << endl;
        return;
    }
    
    executed = YES;
    
    if(Const::debug_level()>4) cerr << Const::modulename(my_id) << endl;
    
//    invalidate_all(); // invalidate all data that the module has created
    // during last execution cycle.
    
    if ( is_inhibited() ) {
        return;
    }
    
    curr_time = samba->get_time();
    
    if ( valid_conds() )
          transform();
    prev_time = curr_time;
    if(Const::debug_level()>4) 
        cerr << Const::modulename(my_id) << " EXECUTED " << endl;
    return;
}

//-------------------------------------------------------------
//
//     Methods of class ModuleList
//
//-------------------------------------------------------------




ModuleList::ModuleList(Module *newmodule, ModuleList *newnext) {

     module = newmodule;
     next = newnext;
}

ModuleList::~ModuleList(){
//     fprintf(stderr,"Deleting listnode of module %s...\n",modulename(module->GetId()));
     if(next != NULL)
          delete next;
//     fprintf(stderr,"Listnode of module %s deleted!\n",modulename(module->GetId()));
 
}


// this method adds nodes and modules to the list, 
// "first in last out" ie. first added module is
// always last in the list
int ModuleList::add_module(Module *newmodule)       // add a node and a module to the list 
{
     if(Const::debug_level_startup())
          fprintf(stderr,"adding module %s...\n",
                  Const::modulename(newmodule->GetId()));
  
     if(module != NULL){ // if this is the first module to add
          next = new ModuleList(module,next); // create a new node to the list
     }
     module = newmodule;

     return 0;
}
