// -*- mode:C++ -*-
// File:   Module.h
// Author: Jussi Pajala 3.3.1997,  jussip@tks.oulu.fi
//
// this file declares class for modules and modulelist

#ifndef MODULE_H
#define MODULE_H 1

#include "EmptySamba.h"
#include "icomm.h"

class EmptySamba; // declared in EmptySamba.h

class ModuleList; // this is declared later in this file

class Module
{
public:
    Module();
    virtual ~Module();
    // Init function should be redeclared for almost any new module
    // because module connections should be created in init.
    virtual void Init(EmptySamba *sambapointer); 
    virtual void reset(){;};
    virtual void execute();
     
    FLAG is_inhibited()    { return inhibited;};
    virtual void set_inhibited()   { inhibited = YES; }; // stop the module!!
    virtual void reset_inhibited() { inhibited = NO;  };
    virtual void reset_executed()  { executed = NO;   invalidate_all();};
    ID GetId()             { return my_id;    };
     
protected:
     ID      my_id;
     FLAG    inhibited;
     FLAG    executed;
          
     int   prev_time;   // time of server and interface, tenths of a second
     int   curr_time;

     EmptySamba *samba; // pointer to main module of the control system
     
     virtual void check_executed(){ if(!executed) execute(); } ;
     
     virtual int transform()= 0;
     virtual void invalidate_all()=0;//this must be implemented in all modules
     virtual FLAG valid_conds(){ return YES;}; // this should be redifined
     // if there is conditions when the module cannot be executed

};



// This class declares list for modules
class ModuleList
{

public:
     ModuleList(Module *newmodule = (Module*)NULL, 
                ModuleList *newnext = (ModuleList*)NULL);
     ~ModuleList();
     
     int add_module(Module *newmodule);      // add node and module to the list 
     ModuleList *GetNext() { return next; } // get pointer to the next node of the list
     Module *GetModule() { return module; } // get module pointer
     
protected:
     ModuleList *next;  // next node of the list
     Module *module;    // module that the node is refering to
};

#endif























