#include "RoundBuffer.h"

template<class S> RoundBuffer<S>::RoundBuffer(int number_of_components)
{
     number = 1+number_of_components;
     elements = new S[number];
     next_to_pop = next_to_push = 0;
     
     pthread_mutex_init(&lock,(pthread_mutexattr_t*)NULL);
}

// returns 1 if OK zero on failure
template<class S> int RoundBuffer<S>::pop(S* elem)
{
     pthread_mutex_lock(&lock);
     if(next_to_pop == next_to_push){
          pthread_mutex_unlock(&lock);
          return 0;
     }
     
     *elem = elements[next_to_pop];
     inc(next_to_pop);
     pthread_mutex_unlock(&lock);
     return 1;
}

template<class S> void RoundBuffer<S>::push(S* elem)
{
     pthread_mutex_lock(&lock);
     elements[next_to_push] = *elem;
     inc(next_to_push);
          
     // if buffer was full and we just wrote new element over old one
     // increase next_to_pop to prevent reading a new instead of an old element
     if(next_to_push == next_to_pop)
          inc(next_to_pop);
     
     pthread_mutex_unlock(&lock);
}

template<class S> void RoundBuffer<S>::push_to_next(S* elem)
{
     pthread_mutex_lock(&lock);
     dec(next_to_pop);

     if(next_to_pop == next_to_push) 
          dec(next_to_push);
     // put element to the first place
     elements[next_to_pop] = *elem;
     pthread_mutex_unlock(&lock);
}


template<class S> void RoundBuffer<S>::inc(int &counter)
{
     counter++;
     if(counter >= number) counter = 0;
}

template<class S> void RoundBuffer<S>::dec(int &counter)
{
     counter--;
     if(counter < 0) counter = number - 1;
}


