// -*- mode:C++ -*-
// Class RoundBuffer
// Purpose of this class is to implement easy-to-use roundbuffer or
// first-in-first-out buffer. This class is usable with different
// types because of use of templates. Whenever new implementation of this 
// class is constructed, implementator (user) must tell number of
// components to the constructor of this class. In the constructor
// memory is reserved to hold elements of the buffer.
// 
// push function is used to push elements to the buffer. when buffer is full
//     the oldest element in the buffer is overwrited
//
// pop is used to get the first element from buffer. Pop returns 1 if success
//     and zero on failure. Be careful when using pop: when there is no 
//     elements in the buffer nothing can be returned!
//
// push_to_next is used to break down the FIFO-order. when push to next is 
//     used, next element to pop is the last element that was pushed with
//     this function. when buffer is full and this function is used, it
//     overwrites the newest element pushed with push()

#ifndef _ROUNDBUFFER_H_
#define _ROUNDBUFFER_H_

#include <pthread.h>

template<class S>
class RoundBuffer
{
public:
     RoundBuffer(int number_of_components);
     int pop(S*);
     void push(S*);
     void push_to_next(S*);
protected:
     void inc(int &counter);
     void dec(int &counter);

     pthread_mutex_t lock;
     
     int number;
     int next_to_pop;
     int next_to_push;
     S* elements;
};

#endif
