// -*- mode:C++ -*-
/* file    : Samba.h
 * includes: the main class of the control system
 */
#ifndef _SAMBA_H
#define _SAMBA_H 1

#include "EmptySamba.h"
#ifdef VIS
#include "Visual.h"
#endif

// Actuators
#include "Ac_base.h"
#include "Ac_kick.h"
#include "Ac_supply.h"
// Arbiters
#include "Arbiter.h"
// Markers
#include <Ma/Ma_goal.h>
#include <Ma/Ma_me.h>
#include <Ma/Ma_ball.h>
#include <Ma/Ma_teammate.h>
#include <Ma/Ma_opponent.h>
#include <Ma/Ma_defloc.h>

// AT
#include <Ma/Ma_offside.h>
#include <Ma/Ma_special_loc.h>
// Virtual sensors
#include <Vs/Vs_referee.h>
#include <Vs/Vs_team_audio.h>
// Behaviors
#include "Be_move.h"
#include "Be_kick.h"
#include "Be_jumpto_defloc.h"
#include "Be_shout.h"
// Sensor module
#include "Sensor.h"

#include "interface.h"

class Samba:public EmptySamba {
public:
     Samba();
     virtual ~Samba();
     void tick();
     void reset();

     SYSTEM_STATE state() {return my_state;}

     Module* get_pointer( ID modulename );
     int get_time() { return curr_time; }
     
protected:
    void inhibit_modules(SYSTEM_STATE);
    void check_sensor_timeout();
    
    // here we declare all the modules.
    Sensor         sensor;
    Ac_base        ac_base;
    Ac_kick        ac_kick;
    Ac_supply      ac_supply;
    Arbiter        arbiter;
    
    Vs_referee     vs_referee;
    Vs_team_audio  vs_team_audio;
    
    Ma_goal        ma_goal;
    Ma_ball        ma_ball;
    Ma_me          ma_me;
    Ma_teammate    ma_teammate;
    Ma_opponent    ma_opponent;
    Ma_defloc      ma_defloc;
    Ma_special_loc ma_specloc;
// AT
     Ma_offside    ma_offside;
     
    
    // behaviors
    Be_move       be_move;
    Be_kick       be_kick;
    
    // supplementary behaviors
    Be_jumpto_defloc be_jumpto_defloc;
    Be_shout be_shout;
    
#ifdef VIS
    Visual         visual;
    int            Visual_execute; // this should be removed to class const
#endif
    // The first node of modulelist.
    // All modules must be in the list!
    ModuleList modulelist;

    SYSTEM_STATE  my_state;
    
    int curr_time; // time of this execution cycle
    int prev_time; // time of the previous exec cycle

    int last_sensor_information;
    int connections;
};

#endif

/* end of file */



