/* file    : Sensor.cc
 * includes: functions for the Object sensor class
 */
//#include <String.h>
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
//#include <signal.h>


#include "decls.h"
#include "Sensor.h"

#include "interface.h"

extern Interface interface;

Sensor::Sensor ()
{
     if(Const::debug_level_startup()) fprintf(stderr,"Starting Sensor... \n");
     
     my_id = SENSOR_ID;
     inhibited = NO;
     prev_time = 0;
     curr_time = 0;
     
     if(Const::debug_level_startup()) cerr <<  "Sensor: CONSTRUCTOR executed" << endl;
}


Sensor::~Sensor()
{
#ifdef TRACE
     trc_save (my_id);
     trc_clear (my_id);
#endif
     
     // DEBUG 
     if(Const::debug_level_exit()) cerr << "Sensor: DESTRUCTOR executed" << endl;
}


// valid conds are satisfied always
FLAG Sensor::valid_conds()
{
     return YES;  // sensor conditions are always valid!
}

void Sensor::convert_vel_vect(Vector &pos, Vector &vel, 
                              float d_dir, float d_dist)
{
     vel = pos;
     vel.turn(d_dir);
     vel.lengthen(d_dist);
     vel = vel - pos;
}


void Sensor::read_interface_flags()
{
     int i;
     
     // read all the flags
     // interface.get_flag(FLAGTYPES flag,float &dir, float &dist, float &d_dir, float &d_dist, int &valid);
     for( i = 0; i < NFLAGS; i++){
          float dir,dist,d_dir, d_dist;
          int valid;
          
          interface.get_flag((FLAGTYPES)i, dir, dist,
                             d_dir, d_dist, valid);
          flags[i].valid = valid;
          if(valid){
               flags[i].time = curr_time;
               flags[i].flag = (FLAGTYPES)i;
               flags[i].pos.set_dir_len(dir,dist);
               convert_vel_vect(flags[i].pos, flags[i].vel, d_dir, d_dist);
          }
          
     }
}

void Sensor::read_interface_lines()
{
     // read all the lines
     int i;
     for(i=0; i<NLINES; i++){
          float dir, d_dir, dist, d_dist;
          int valid;
          
          interface.get_line((LINETYPES)i, dir, dist, d_dir, 
                             d_dist, valid);
          lines[i].valid = valid;
          if(valid){
               lines[i].line = (LINETYPES)i;
               lines[i].dir = dir;
               lines[i].pos.set_dir_len( 0.0, dist );
               // line relative velocity is not calculated because
               // the server doesn't report it
          }
// line dir is quite hard to understand, so here is some explanations
/*

                    _
                    /| line end point
                   /                
                  /
                 /
                /\ alfa
   player O---------------> facedir
              /
             /
              line start point
           
          
   now alfa is angle to facedir, so here now it is positive.
   line vector can be tought to start at a point where line
   is nearest to player and end at a point where line is furhest

   when tought this way, alfa can never be >90 degrees or <-90 degrees
   ( i.e. - 1/2 * pi <= alfa <= 1/2 * pi )
   
   line distance is always distance to crossing of line and face dir vectors
   this is how server gives line angles
*/
     }
}

void Sensor::read_interface_players()
{
// read players
     float dir[MAX_SEEN_PLAYERS], dist[MAX_SEEN_PLAYERS], d_dir[MAX_SEEN_PLAYERS], d_dist[MAX_SEEN_PLAYERS], head[MAX_SEEN_PLAYERS];
     int unum[MAX_SEEN_PLAYERS], valid[MAX_SEEN_PLAYERS];
     int i;
     
     int robs = 0;

     for(i=0; i<MAX_SEEN_PLAYERS; i++) valid[i] = 0;
     
     interface.get_team(dir,dist, d_dir, d_dist, head, unum, 
                        valid);
     for (i=0; i<NFRIENDS; i++)
          if(valid[i]){
               if(robs > 20){ cerr << __FILE__ << "  " << __LINE__<< endl; exit(1);}
               if(Const::debug_level()>8) cerr << "  " << __LINE__ << ": Team-mate " << endl;
               players[robs].unum = unum[i];
               players[robs].time = curr_time;
               players[robs].facedir = head[i];
               players[robs].side = Side_L;
               players[robs].valid = valid[i];
               players[robs].pos.set_dir_len(dir[i], dist[i]);
               convert_vel_vect(players[robs].pos, players[robs].vel, 
                                   d_dir[i], d_dist[i]);
               robs++;
          }
     //cerr << "robs: " << robs << " " << __LINE__;
     
     for(i=0; i<MAX_SEEN_PLAYERS; i++) valid[i] = 0;
     
     interface.get_uk_team(dir, dist, d_dir, d_dist, head,
                           valid);
     for (i=0; i<NUKFRIENDS; i++)
          if(valid[i]){
               if(robs > 20){ cerr << __FILE__ << "  " << __LINE__ << endl; exit(1);}
               players[robs].unum = 0;
               players[robs].time = curr_time;
               players[robs].facedir = head[i];
               players[robs].side = Side_L;
               players[robs].valid = valid[i];
               players[robs].pos.set_dir_len(dir[i], dist[i]);
               convert_vel_vect(players[robs].pos, players[robs].vel, 
                                   d_dir[i], d_dist[i]);
               robs++;
          }
     //cerr << "robs: " << robs << " " << __LINE__;
     
     for(i=0; i<MAX_SEEN_PLAYERS; i++) valid[i] = 0;
     
     interface.get_opp(dir,dist, d_dir, d_dist, head, unum, valid);
     for(i=0; i<NOPPS; i++)
          if(valid[i]){
              if(robs > 20){ cerr << __FILE__ << "  " << __LINE__ << endl; exit(1);}
              if(Const::debug_level()>8) cerr << "  " << __LINE__ << " Opponent " << endl;
              players[robs].unum = unum[i];
              players[robs].time = curr_time;
              players[robs].facedir = head[i];
              players[robs].side = Side_R;
              players[robs].valid = valid[i];
              players[robs].pos.set_dir_len(dir[i], dist[i]);
              convert_vel_vect(players[robs].pos, players[robs].vel, 
                               d_dir[i], d_dist[i]);
              robs++;
          }
     //cerr << "robs: " << robs << " " << __LINE__;
     
     for(i=0; i<MAX_SEEN_PLAYERS; i++) valid[i] = 0;
     interface.get_uk_opp(dir,dist, d_dir, d_dist, head, valid);
     for(i=0; i<NUKOPPS; i++)
          if(valid[i]){
               if(robs > 20){
                    cerr << __FILE__ << "  " << __LINE__ << endl;
                    //cerr << "robs: " << robs << endl;
                    exit(1);
               }
               if(Const::debug_level()>8) cerr << "  " << __LINE__ << " Uk-opponent " << endl;
               players[robs].unum = 0;
               players[robs].time = curr_time;
               players[robs].facedir = head[i];
               players[robs].side = Side_R;
               players[robs].valid = valid[i];
               players[robs].pos.set_dir_len(dir[i], dist[i]);
               convert_vel_vect(players[robs].pos, players[robs].vel, 
                                   d_dir[i], d_dist[i]);
               robs++;
          }
//     cerr << " " << robs;
     for(i=0; i<MAX_SEEN_PLAYERS; i++) valid[i] = 0;
     
     interface.get_uk(dir,dist, d_dir, d_dist, head, valid);
     for(i=0; i<NUNKNOWNS; i++)
          if( valid[i] ){
              if(robs > 20){ 
                  interface.print_all_robots_and_exit();
                  cerr << __FILE__ << "  " << __LINE__ 
                       <<  " Too many robots..." << endl;exit(1);
              }
              if(Const::debug_level()>8) cerr << "  " << __LINE__ << " Unknonwn " << endl;
              players[robs].unum = 0;
              players[robs].time = curr_time;
              players[robs].side = Side_none;
              players[robs].facedir = head[i];
              players[robs].valid = valid[i];
              players[robs].pos.set_dir_len(dir[i], dist[i]);
              convert_vel_vect(players[robs].pos, players[robs].vel, 
                               d_dir[i], d_dist[i]);
              robs++;
          }
//     cerr << " " << robs << endl;
     
     
     if(Const::debug_level() > 8) cerr << "Number of seen players: " << robs << endl;
     
     // fill the rest of plrs array with players whose unum is greater than 11.
     // so we can check if the array node is empty
     for( i = robs; i < MAX_SEEN_PLAYERS; i++){
         players[i].unum = 0;          players[i].time = 0;
          players[i].side = Side_none; players[i].facedir = 0;
          players[i].valid = 0;        players[i].pos.reset();
          players[i].vel.reset();
                      
     }
}

void Sensor::read_interface_goals()
{
          
     float dir, // information of a goal
          dist,  
          d_dir,
          d_dist;
     int valid;

     
     // read opp_goal sensor
     interface.get_opp_goal(dir, dist, d_dir, d_dist, valid);
     if(valid){
          // relative position
          opp_goal.pos.set_dir_len(dir, dist);
          // relative velocity
          convert_vel_vect(opp_goal.pos, opp_goal.vel, d_dir, d_dist);
          opp_goal.time = curr_time;
          opp_goal.valid = 1;
     }
     else opp_goal.valid = 0; // not valid when below zero
          
     // read own_goal sensor
     interface.get_own_goal(dir, dist, d_dir, d_dist, valid);
     if(valid){
          // relative position
          own_goal.pos.set_dir_len(dir, dist);
          // relative velocity
          convert_vel_vect(own_goal.pos, own_goal.vel, d_dir, d_dist);
          own_goal.time = curr_time;
          own_goal.valid = 1;
     } else own_goal.valid = 0;
}

void Sensor::read_interface_ball()
{
     float ball_dir, // ball's information
          ball_dist,
          ball_d_dir,
          ball_d_dist;
     int ball_valid;
     
     interface.get_ball(ball_dir, ball_dist, ball_d_dir, ball_d_dist, ball_valid);
     
     if(ball_valid){
          ball.pos.set_dir_len(ball_dir,ball_dist);
          convert_vel_vect(ball.pos, ball.vel, ball_d_dir, ball_d_dist);
          ball.time = curr_time;
          ball.valid = 1;
     }
     else ball.valid = 0; // not valid when below zero
}

void Sensor::read_interface_audio()
{
     int i;
     
     for(i = 0; i < MAX_HEARD_MSG; i++){
         if( !interface.get_heard_msg(audio[i]) ){
             //message[i].valid = 0;
             //message[i].message[0] = '\0';
         }
     }
}

void Sensor::read_flag_sensors(Flag_sensor *to)
{
    check_executed();
    memcpy(to, flags, NFLAGS*sizeof(Flag_sensor));
}

void Sensor::read_line_sensors(Line_sensor *to)
{
    check_executed();
    memcpy(to, lines, NLINES*sizeof(Line_sensor));
}
     

void Sensor::read_ball_sensor(Ball_sensor &to)
{
     check_executed();
     to = ball;
}

void Sensor::read_player_sensor(Player_sensor *to){
     check_executed();
     memcpy(to, players, MAX_SEEN_PLAYERS*sizeof(Player_sensor));
}
     

void Sensor::read_goal_sensors(Goal_sensor &opp, Goal_sensor &own){
     check_executed();
     opp = opp_goal;
     own = own_goal;
}


void Sensor::read_audio_sensor(Audioinfo *to)
{
     check_executed();
     for(int i = 0; i < MAX_HEARD_MSG; i++)
          if(audio[i].valid){
               to[i] = audio[i];
          } else {
               // prevent useless copying of data
               to[i].valid = 0;
          }
}

void Sensor::read_sense_body(Sense_body &to){
      check_executed();
      to = sensebody;
}


// invalidate_all() function invalidates all sensor data
// to ensure that false data is not relayed outside this module
void Sensor::invalidate_all()
{
     int i;
          
     some_valid = NO;
     for(i = 0; i < MAX_HEARD_MSG; i++)
          audio[i].valid = 0;
     
     ball.valid = 0;
     
     for(i = 0; i < NFLAGS; i++)
          flags[i].valid = 0;
     
     own_goal.valid = 0;
     opp_goal.valid = 0;
     
     for(i = 0; i < NLINES; i++)
          lines[i].valid = 0;
     
     for(i = 0; i < MAX_SEEN_PLAYERS; i++)
          players[i].valid = 0;
     
     sensebody.valid = 0;
}

    

int Sensor::transform ()
{
     // read_sensors
     // at first check if there is new sensor information   
     if(interface.is_any_valid()){
          // read sensors and send signal if sensor's object is valid
          read_interface_ball();
          read_interface_goals();
          read_interface_lines();
          read_interface_flags();
          read_interface_players();
          // read all the other sensors too.... if any
     } else {
          // can not do anything in here!! 
     }
     // Audio sensors must be read at every time
     read_interface_audio();

     // And sensebody information hopefully arrives at every cycle...
     interface.get_sense_body(sensebody);
     
     // read robot's current position on the field (measured or approximated)
     my_pos.set_xy(interface.get_x(),interface.get_y());
     my_vel.set_xy(interface.get_xv(), interface.get_yv());
     my_facedir = interface.get_angle();
     
     // if server sent new information, there is some valid data so put
     // some_valid to yes
     if(interface.is_any_valid()) some_valid = YES;
     else some_valid = NO;
         
     // data must be invalidated after it is once read. This ensures that
     // we don't treat it as new data at next cycle
     interface.invalidate_all();
     // release sensor data, new data may be written(in interface.cc)after this
     // but READING MUST NOT BE DONE after this
     interface.unlock_sensor_data();
     
//     char a[2] = "";
//     supply_cmd_out.signal.write(NONE,0,0,&(a[0]),my_id);
     //supply_cmd_out.send(my_id);

     if(Const::debug_level() > 8) cerr << endl;
     
     return 0;
}

/* end of file */


