// -*- mode:C++ -*-

// file    : Se_object.h
//includes: class declaration for the Object sensor
//

#ifndef _SENSOR_H
#define _SENSOR_H 1
#include "fieldobject.h"
#include "Module.h"
#include "Vector.h"
struct Ball_sensor
{
     int time;
     int valid;
     Vector pos;
     Vector vel;
};

typedef Ball_sensor Goal_sensor;


struct Player_sensor
{
     int time;
     int valid;
     int unum;
     Side side;
     Vector pos;
     Vector vel;
     float facedir; // should this be vector?
};

struct Flag_sensor
{
     int time;
     int valid;
     FLAGTYPES flag;
     Vector pos;
     Vector vel;
};

struct Line_sensor
{
     int time;
     int valid;
     LINETYPES line;
     float dir;
     Vector pos;
};



class Sensor:public Module {
public:
    Sensor ();
    ~Sensor ();
    
    // following two virtual methods are redeclared here
    virtual void reset () {;}// nothing can be resetted
    
    // set inhibited function is over redeclared here to
    // prevent stopping this module.
    void set_inhibited()   { inhibited = NO; }; // cannot stop the module!!

    FLAG is_any_valid(){ check_executed(); return some_valid; }
    void read_flag_sensors(Flag_sensor *to);
    void read_line_sensors(Line_sensor *to);
    void read_ball_sensor(Ball_sensor &to);
    void read_player_sensor(Player_sensor *to);
    void read_goal_sensors(Goal_sensor &opp, Goal_sensor &own);
    void read_audio_sensor(Audioinfo *to);
    void read_sense_body(Sense_body &to);
    
protected:
    FLAG   valid_conds();
    int    transform();
    void convert_vel_vect(Vector &pos, Vector &vel, 
                          float d_dir, float d_dist);
    
    void read_interface_ball();
    void read_interface_goals();
    void read_interface_lines();
    void read_interface_flags();
    void read_interface_players();
    void read_interface_audio();
    
    void invalidate_all(); // invalidates all sensor data
    
    FLAG some_valid;
    Audioinfo audio[MAX_HEARD_MSG];
    Ball_sensor ball;
     Flag_sensor flags[NFLAGS];
    Goal_sensor own_goal;
    Goal_sensor opp_goal;
    Line_sensor lines[NLINES];
    Player_sensor players[MAX_SEEN_PLAYERS];
    Sense_body sensebody; // struct Sense_body defined in fieldobject.h
    Vector my_pos, my_vel;
    float my_facedir;
};

#endif

/* end of file */
