#include "State_machine.h"
#include "State_machine_gk.h"

#include "Arbiter.h"
#include "Cons2.h"
#include <Am/Moveto_map.h>
#include <Am/Kickto_map.h>

#include "iostream.h"
#include "stdlib.h"

int state_machine_date = 0;

// returns 1 if kick impulse is towards opponent side
// else returns 0
int good_g_imp(Vector imp, float heading)
{

    imp.turn(heading);
    if(fabs(imp.get_dir()) > M_PI/2.0 && imp.get_len() < 1.0) return 0;
    return 1;
}

// State machine saves all its states to this list structure
State_list::State_list(Prim_state *newstate, State_list *newnext) {
    state = newstate;
    next = newnext;
}

State_list::~State_list(){
    if(next != NULL)
        delete next;
}


// this method adds nodes and states to the list, 
// "first in last out" ie. first added state is
// always last in the list
int State_list::add_state(Prim_state *newstate)
{
    if(state != NULL){ // if this is the first state to add
        next = new State_list(state,next); // create a new node to the _list
    }
    state = newstate;
    
    return 0;
}



// Prim_state is Super class for all state machine classes
Prim_state::Prim_state(Arbiter *arbiterpointer)
{
    arbiter = arbiterpointer;
    state_on = 0;
    strcpy(name,"NAME NOT INITIALIZED!");
}

void Prim_state::print_state_name()
{
    cerr << name << endl;
}

void Prim_state::reset()
{ 
    state_on = 0; 
}

// State_machine is superclass for all state machines
State_machine::State_machine(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer)
{
    arbiter = arbiterpointer;
    state_list = new State_list;

    strcpy(name,"STATE_MACHINE");
    curr_state = NULL;
}

State_machine::~State_machine()
        
{
    // first delete modules created by new 
    State_list *temp;
    temp = state_list;
    while(temp != NULL){
        delete temp->get_state();
        temp = temp->get_next();
    }
    // then delete first node of the list and the rest of the list will be deleted by
    // the list itself
    delete state_list;
}

int State_machine::valid_conds()
{
    return 1;
}


int State_machine::execute()
{
    // if conditions is not valid, execution can not be done
    // reset all states and return 1
    if(!valid_conds()){
        state_on = 0;
        State_list *temp = state_list;
        while(temp){
            temp->get_state()->reset();
            temp = temp->get_next();
        }
        return 1;
    }
    state_on = 1;
        

    // try to execute the state which was executed succesfully during 
    // previous round
    if( curr_state != NULL ){
        // returns 0 when done succesfully
        // if so, return from state machine, all is done for this round
        if( !curr_state->execute() ) return 0;
        else curr_state->reset(); // clear state_on flag
    }
    
    // copy pointer of the state list to temporary variable and check the list
    State_list *temp = state_list;
    if( !temp ){
        cerr << "State list is empty!" << endl;
        exit(1);
    }
    
    // start from the beginning of the list and find the first state 
    // that is valid
    curr_state = temp->get_state();
    
    while ( !curr_state->valid_conds() || curr_state->execute() ){
        temp = temp->get_next();
        if(temp == NULL){
            curr_state = NULL;
            if(Cons2::debug_level()>1)
                cerr << "Player " << Cons2::me_unum() << " " << name << " " 
                     << __LINE__ << " Nothing could be done!" << endl;
            return 1;
        }
        
        curr_state = temp->get_state();
        if(curr_state == NULL ){
            // return 1 if nothing could be done
            cerr << name << " " << __LINE__ << " Nothing could be done!" << endl;
            return 1;
        }
    }
    // put on state_on flag to tell that the state was executed succesfully
    curr_state->mark_state_on();
    
    // return 0 when done
    return 0;
}



// 
// This is the main state machine
//
Main_state_machine::Main_state_machine(Arbiter *arbiterpointer)
        :State_machine(arbiterpointer){
    strcpy(name, "Main_state_machine");
        
    if(Cons2::me_unum() == GOAL_KEEPER_UNUM) 
        state_list->add_state((Prim_state*)new Goal_keeper(arbiter) );
    else{
        state_list->add_state((Prim_state*)new Turnto_ball(arbiter) );     // the least important
        state_list->add_state((Prim_state*)new Goto_defloc(arbiter) );
        state_list->add_state((Prim_state*)new Avoid_offside_sm(arbiter));
/*    state_list->add_state((Prim_state*)new Goto_field(arbiter) );*/
        state_list->add_state((Prim_state*)new Catch_ball(arbiter) );

//        state_list->add_state((Prim_state*)new PanicKick(arbiter) );

        state_list->add_state((Prim_state*)new KickFar(arbiter) );
/*        state_list->add_state((Prim_state*)new Dribble(arbiter) );

          state_list->add_state((Prim_state*)new Pass(arbiter) );*/
        state_list->add_state((Prim_state*)new Make_goal(arbiter) );

        state_list->add_state((Prim_state*)new Pass_close(arbiter) );
        state_list->add_state((Prim_state*)new Make_close_goal(arbiter));


        state_list->add_state((Prim_state*)new Lookaround(arbiter) );
        state_list->add_state((Prim_state*)new Special_situation(arbiter)); // the most important
    }
        
// in the beginning current state is special situation state ie. the state in
// the first node of the State_list
        curr_state = state_list->get_state();
}


// uses Kickto_goal action map if ball will be in goal under 1.5 seconds
Make_close_goal::Make_close_goal(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Make_close_goal");
}

int Make_close_goal::execute()
{
    if(!valid_conds()) return 1;
    Vector best_vector;
    Kickto_map *am_kick = arbiter->be_kick->read(MAKE_GOAL);// MAKE_GOAL, PAS MAP WAS, DRIBBLE,
    
    // if time to goal takes too long, don't kick!!
    if( am_kick->find_best_action(best_vector) ){
        if(am_kick->weight2time( am_kick->get_best_wgt() ) 
           > 15.0 /*Const2::close_goal_time()*/) return 1;
        
        Kick_cmd temp;
        temp.imp = best_vector;
        temp.g_vel = 
            am_kick->mix_vel_after_kick(arbiter->ball.mix_vel, best_vector);
        temp.g_vel.turn(arbiter->me.heading);
        temp.valid = 1;
        if(!good_g_imp(temp.imp, arbiter->me.heading)) return 1;
        if(Cons2::debug_level()>1)
            cerr << "Player " << Cons2::me_unum() << " kicking CLOSE_GOAL" << endl;

        arbiter->kick( temp );
        return 0;
    }

    return 1;
}


// uses kickto_goal actionmap
Make_goal::Make_goal(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Make_goal");
}

int Make_goal::execute()
{
    if(!valid_conds()) return 1;
    Vector best_vector;
    Kickto_map *am_kick = arbiter->be_kick->read(MAKE_GOAL);// MAKE_GOAL, PASS, DRIBBLE,
    if( am_kick->find_best_action(best_vector) ){
        Kick_cmd temp;
        temp.imp = best_vector;
        temp.g_vel = 
            am_kick->mix_vel_after_kick(arbiter->ball.mix_vel, best_vector);
        temp.g_vel.turn(arbiter->me.heading);
        temp.valid = 1;
        if(!good_g_imp(temp.imp, arbiter->me.heading)) return 1;
        if(Cons2::debug_level()>1)
            cerr << "Player " << Cons2::me_unum() << " kicking GOAL" << endl;

        arbiter->kick( temp );
        return 0;
    }

    return 1;
}

// uses kickto_teammate actionmap if time to teammate is under 1.5 seconds
Pass_close::Pass_close(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Pass_close");
}

int Pass_close::execute()
{
    if(!valid_conds()) return 1;
    Vector best_vector;
    Kickto_map *am_kick = arbiter->be_kick->read(PASS);// MAKE_GOAL, PASS, DRIBBLE,
    if( am_kick->find_best_action(best_vector) ){
        if(am_kick->weight2time( am_kick->get_best_wgt() ) 
           > 15.0 ) return 1;

        Kick_cmd temp;
        temp.imp = best_vector;
        temp.g_vel = 
            am_kick->mix_vel_after_kick(arbiter->ball.mix_vel, best_vector);
        temp.g_vel.turn(arbiter->me.heading);
        temp.valid = 1;
        if(!good_g_imp(temp.imp, arbiter->me.heading)) return 1;
        if(Cons2::debug_level()>1)
            cerr << "Player " << Cons2::me_unum() << " kicking CLOSE_PASS" << endl;

        arbiter->kick( temp );
        return 0;
    }
    return 1;
}

// uses kickto_teammate actionmap 
Pass::Pass(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Pass");
}

int Pass::execute()
{
    if(!valid_conds()) return 1;
    Vector best_vector;
    Kickto_map *am_kick = arbiter->be_kick->read(PASS);// MAKE_GOAL, PASS, DRIBBLE,
    if( am_kick->find_best_action(best_vector) ){
        Kick_cmd temp;
        temp.imp = best_vector;
        temp.g_vel = 
            am_kick->mix_vel_after_kick(arbiter->ball.mix_vel, best_vector);
        temp.g_vel.turn(arbiter->me.heading);
        temp.valid = 1;
        if(!good_g_imp(temp.imp, arbiter->me.heading)) return 1;
        arbiter->kick( temp );
        if(Cons2::debug_level()>1)
            cerr << "Player " << Cons2::me_unum() << " kicking PASS" << endl;

        return 0;
    } 
    
    return 1;
}

// uses kickto_near actionmap
Dribble::Dribble(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Dribble");
}

int Dribble::execute()
{
    if(!valid_conds()) return 1;
    
    static int counter = 0;
    if(state_on){
        counter++;
        cerr << counter << " " ;
        
        if(counter>1){
            if(counter>5) counter = 0;
            return 0;
        }
    }
    else counter = 0;
    Vector best_vector;
    Kickto_map *am_kick = arbiter->be_kick->read(DRIBBLE);// MAKE_GOAL, PASS, DRIBBLE,
    if( am_kick->find_best_action(best_vector) ){
        Kick_cmd temp;
        temp.imp = best_vector;
        temp.g_vel =  
            am_kick->mix_vel_after_kick(arbiter->ball.mix_vel, best_vector);
        temp.g_vel.turn(arbiter->me.heading);
        temp.valid = 1;
        if(!good_g_imp(temp.imp, arbiter->me.heading)) return 1;
        arbiter->kick( temp );
        if(Cons2::debug_level()>1)
            cerr << "Player " << Cons2::me_unum() << "kicking  DRIBBLE" << endl;
        return 0;
    }
    
    return 1;
}

// uses kickto_far action map 
KickFar::KickFar(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"KickFar");
}

int KickFar::execute()
{
    if(!valid_conds()) return 1;
    Vector best_vector;
    Kickto_map *am_kick = arbiter->be_kick->read(FAR);
    
    if( am_kick->find_best_action(best_vector) ){
        Kick_cmd temp;
        temp.imp = best_vector;
        temp.g_vel =  
            am_kick->mix_vel_after_kick(arbiter->ball.mix_vel, best_vector);
        temp.g_vel.turn(arbiter->me.heading);
        temp.valid = 1;
        if(!good_g_imp(temp.imp, arbiter->me.heading)) return 1;
        arbiter->kick( temp );
        if(Cons2::debug_level()>1)
            cerr << "Player " << Cons2::me_unum() << " kicking FAR" << endl;
        return 0;
    }
    return 1;
}

// doesn't use any action map, just calculates vector that points to 
// opponent goal
PanicKick::PanicKick(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"PanicKick");
    just_executed = 0;
}

int PanicKick::execute()
{
    if(!valid_conds()) return 1;
    just_executed = 1;
    Vector best_vector;

    best_vector.set_dir_len(arbiter->opp_goal.loc.center.get_dir(), 100.0);
    
    Kick_cmd temp;
    temp.imp = best_vector;
    temp.g_vel = arbiter->ball.mix_vel + Cons2::kick_power_rate()*best_vector;
    temp.g_vel.turn(arbiter->me.heading);
    temp.valid = 1;
    if(!good_g_imp(temp.imp, arbiter->me.heading)){
        cerr << __FILE__ << " " << __LINE__ << " we should never come here!" << endl;
        assert(0);
        return 1;
    }
    arbiter->kick( temp );
    if(Cons2::debug_level()>1)
        cerr << "Player " << Cons2::me_unum() << " kicking in PANIC" << endl;
    return 0;
}

Goto_defloc::Goto_defloc(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Goto_defloc");
}

int Goto_defloc::execute()
{
    if(Cons2::me_unum()==1) cerr << "GOALKEEPER DEFLOC!" << endl;
    
    if(!valid_conds()) return 1;
    Vector vect_defloc;
    Moveto_map *am_defloc = arbiter->be_move->read(GOTO_DEFLOC);
    if( am_defloc->find_best_action(vect_defloc) ){
        Move_cmd temp;
        temp.dir = vect_defloc.get_dir();
        temp.spd = vect_defloc.get_len();
        temp.valid = 1;
        arbiter->move(temp);
        return 0;
    }
    return 1;
}
Turnto_ball::Turnto_ball(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
        strcpy(name,"Turnto_ball");
}

int Turnto_ball::execute()
{
    if(!valid_conds()) return 1;
    Move_cmd temp;
    temp.dir = arbiter->ball.loc.get_dir();
    temp.spd = 0.0;
    temp.valid = 1;
    arbiter->move(temp);
    return 0;
}


Lookaround::Lookaround(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Lookaround");

}


int Lookaround::execute()
{
    // if conditions is not valid, execution can not be done
    // return 1
    if(!valid_conds()) return 1;
    
    // lookaround is quite a primitive function:
    // if new sensordata has arrived we can turn again, else we'll just
    // return
    if(arbiter->is_any_sensor_valid){
        // now we'll command the base actuator to turn
        float visual_angle = Cons2::visual_angle();
        Move_cmd move;
        move.spd = 0.0;
        if(Cons2::me_unum()%2) move.dir = visual_angle*0.9;//0.8;
        else move.dir = -visual_angle*0.9;//0.8;
        move.valid = 1;
        arbiter->move(move); // vel 0.0, turn 0.1
    }
    
    return 0;
}

Catch_ball::Catch_ball(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
        strcpy(name,"Catch_ball");
}

int Catch_ball::execute()
{
    if(!valid_conds()) return 1;

    Vector catch_vector;
    Moveto_map *am_catch;
    if(arbiter->must_avoid_opponents())
        am_catch = arbiter->be_move->read(GOTO_BALL);
    else am_catch = arbiter->be_move->read(GOTO_BALL_NO_OPP_AVOID);
    if( am_catch->find_best_action(catch_vector) ){
        /*if(Cons2::debug_level()){
          static last_send = 0;
          if(arbiter->curr_time > last_send+10){
          am_catch->cpp_send("Catch ball action map");
          last_send = arbiter->curr_time;
          }
          }*/
        Move_cmd temp;
        temp.dir = catch_vector.get_dir();
        temp.spd = catch_vector.get_len();
        temp.valid = 1;
        arbiter->move( temp );
        return 0;
    }
    
    return 1;
}

Avoid_offside_sm::Avoid_offside_sm(Arbiter *arbiterpointer)
        :State_machine(arbiterpointer){
    strcpy(name,"Avoid_offside_state_machine");
    
    state_list->add_state((Prim_state*)new Move_from_offside(arbiter) );     // the least important
    state_list->add_state((Prim_state*)new Offside_lookaround(arbiter) );
}


Move_from_offside::Move_from_offside(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Move_from_offside");
    end_time = -1;
    start_time = -1;
}

int Move_from_offside::execute()
{
    if(!valid_conds()) return 1;

    if(!start_time) {
        start_time = arbiter->curr_time;
        if(Cons2::debug_level())  cerr << "Player " << Cons2::me_unum() << " avoiding offside!" << endl;
    }
    
    Vector move_vector;
    Moveto_map *am_offside;
    
    am_offside = arbiter->be_move->read(GOTO_AVOID_OFFSIDE);
    
    if( am_offside->find_best_action(move_vector) ){
        Move_cmd temp;
        temp.dir =  move_vector.get_dir();
        temp.spd = move_vector.get_len();
        temp.valid = 1;
        arbiter->move( temp );
        return 0;
    }
    end_time = arbiter->curr_time;
    return 1;
}
Offside_lookaround::Offside_lookaround(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Offside_lookaround");
    end_time = -1;
    start_time = -1;
}

int Offside_lookaround::execute()
{
    if(!valid_conds()) return 1;
    if(!start_time) start_time = arbiter->curr_time;
    
    if(arbiter->is_any_sensor_valid){
        // now we'll command the base actuator to turn
        float visual_angle = Cons2::visual_angle();
        static Move_cmd move;
        move.spd = 0.0;
        if(Cons2::me_unum()%2) move.dir = visual_angle*0.9;//0.8;
        else move.dir = -visual_angle*0.9;// -0.8;
        move.valid = 1;
        arbiter->move(move); // vel 0.0, turn 0.8
        return 0;
    } 
    return 0;
}

Turnto_goal::Turnto_goal(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name, "Turnto_goal");
    on = 0;
}

int Turnto_goal::execute()
{
    if(!valid_conds()) return 1;
    
    on++;
    
    Move_cmd temp;
    temp.dir = arbiter->opp_goal.loc.center.get_dir();
    temp.spd = 0.0;
    temp.valid = 1;
    arbiter->move(temp);
    return 0;
}




int Lookaround::valid_conds()
{
    if( arbiter->me.g_cer < 0.5 ) return 1;
    if( arbiter->ball.cer < 0.1 ) return 1;
    return 0;
}

int Pass_close::valid_conds()
{
    // don't pass if near own goal!
    if(arbiter->ball.g_loc.get_x() < -15) return 0;
    
    return arbiter->ball_kickable();
}

int Pass::valid_conds()
{
    // don't pass if near own goal!
    if(arbiter->ball.g_loc.get_x() < -15) return 0;
    
    return arbiter->ball_kickable();
}

int Dribble::valid_conds()
{
//    return 0;
    
    for( int i = 0; i < 11; i++){
        // if any player too close, valid conds not valid!
        if(arbiter->opp_team.player[i].loc.get_len() < 8.0
           && arbiter->opp_team.player[i].cer > 0.1) return 0;
           }
    
    // don't dribble if near own goal!
    if(arbiter->ball.g_loc.get_x() < 0.0) return 0;
    
    return arbiter->ball_kickable();
}

int KickFar::valid_conds()
{
//    if(arbiter->me.heading_cer < 0.99) return 0;
/*    float nearest = 1000.0;
    for( int i = 0; i < 11; i++){
        // seek for the closest player
        if( arbiter->opp_team.player[i].loc.get_len() < nearest
            && arbiter->opp_team.player[i].cer > 0.1) 
            nearest = arbiter->opp_team.player[i].loc.get_len();
    }
    // don't use farkick when not near own goal 
    // and if there is no players near!
    if(arbiter->me.g_loc.get_x() > -20 && nearest > 4.0) return 0;*/
    return arbiter->ball_kickable();
}
int PanicKick::valid_conds()
{
    if(just_executed){
        just_executed = 0;
        return 0;
    }
    return arbiter->ball_kickable();
}

int Make_close_goal::valid_conds()
{
    // don't try to make a goal if too far from goal
    if(arbiter->opp_goal.loc.center.get_len() > 50) return 0;
    return arbiter->ball_kickable();
}

int Make_goal::valid_conds()
{
    // don't try to make a goal if too far from goal
    if(arbiter->opp_goal.loc.center.get_len() > 50) return 0;
    return arbiter->ball_kickable();
}

int Catch_ball::valid_conds()
{
    return arbiter->catch_ball_valid();
}
int Goto_defloc::valid_conds()
{
    // if ball can be kicked, don't go to defloc
    if(arbiter->ball.loc.get_len() < Cons2::kick_max_dist() ) return 0;
    // if defloc is nearer that two meters don't go there
    if(arbiter->defloc.loc.get_len() < 4.0 ) return 0;
    // if ball certainty has dropped a little, take a look towards it
    if(arbiter->ball.cer < 0.6) return 0;

    if(arbiter->catch_ball_valid()/*closest_to_ball()*/){
        /*cerr << "Player " << Cons2::me_unum() 
          << " not going to defloc because closest to ball " << endl;*/
        return 0;
    }
    if(arbiter->opponent_special_situation() && 
       arbiter->ball.loc.get_len() < 11.0 ) return 0;
    
    
    // if going to defloc reduces distance to ball, goto there
    if(fabs(arbiter->ball.loc.angle_to(arbiter->defloc.loc)) < 0.5*M_PI)
        return 1;
    // also goto defloc if distance to ball won't reduce by going there
    // but distance to defloc is greater than 15 meters
    else if(arbiter->defloc.loc.get_len() > 15.0) return 1;

    if(arbiter->me.g_loc.get_x() < -10.0 && arbiter->ball.g_loc.get_x() < 10.0)
        return 1;
    
    if(arbiter->me.g_loc.get_x() - arbiter->ball.g_loc.get_x() > 30.0)
        return 1;
        
    return 0;
}

int Turnto_ball::valid_conds()
{
    if(arbiter->ball.cer < 0.1) return 0;
    if(arbiter->ball.loc.get_dir() < 0.2 && 
        arbiter->ball.cer > 0.9) return 0;
    if(arbiter->ball.loc.get_len() <=Cons2::kick_max_dist()) return 0;
    
//    if(arbiter->closest_to_ball()) return 0;
    // else conditions are valid!
    return 1;
}

Special_situation::Special_situation(Arbiter *arbiterpointer)
        :State_machine(arbiterpointer){
    strcpy(name, "Special_situation_state_machine");
    
    state_list->add_state((Prim_state*)new PanicKick(arbiter) );
    state_list->add_state((Prim_state*)new KickFar(arbiter) );
    state_list->add_state((Prim_state*)new Pass(arbiter) );
    state_list->add_state((Prim_state*)new Make_goal(arbiter) );
    state_list->add_state((Prim_state*)new Turnto_goal(arbiter) );
    state_list->add_state((Prim_state*)new Turnto_ball(arbiter) );
    state_list->add_state((Prim_state*)new Avoid_offside_sm(arbiter) );
    state_list->add_state((Prim_state*)new Goto_specloc(arbiter) );
    state_list->add_state((Prim_state*)new Lookaround(arbiter) );
        
// in the beginning current state is special situation state ie. the state in
// the first node of the State_list
    curr_state = state_list->get_state();
}


int Special_situation::valid_conds()
{
    if(arbiter->referee.game_state == play_on){
        return 0;
    }
    
    return 1;
}

Goto_specloc::Goto_specloc(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer){
    strcpy(name,"Goto_specloc");
}

int Goto_specloc::execute()
{
    static int valid_counter = 0;
    
    if(!valid_conds()){
        valid_counter = 0;
        return 1;
    }
    
    static Move_cmd temp;
    static Vector vect_specloc;
    static Moveto_map *am_specloc;
        
    // check if we are too close to ball
    if(arbiter->ball.loc.get_len() < 
       Cons2::player_size() + Cons2::ball_size() + 0.1 ){
        vect_specloc = arbiter->specloc.loc;
                
        temp.dir = -arbiter->specloc.loc.get_dir();
        normalize_angle(temp.dir);
        temp.spd = 0.3;
        temp.valid = 1;
    } else {
        // use actionmap when far from special location
        am_specloc = arbiter->be_move->read(GOTO_SPECLOC);    
        float max_vel;
        if((arbiter->referee.opp_goals < arbiter->referee.team_goals 
           || arbiter->specloc.loc.get_len() < 4.0) && Cons2::me_unum()>1)
            max_vel = 0.3;
        else max_vel = 0.6;
        am_specloc->filter_speeds_faster_than(max_vel);
        
        if(!am_specloc->find_best_action(vect_specloc) ) {
            // do something else if action not found
            valid_counter = 0;
            return 1;
        }
        
        temp.dir = /*arbiter->specloc.loc.get_dir()*/
            vect_specloc.get_dir();
        temp.spd = /*0.3;*/ vect_specloc.get_len();
        temp.valid = 1;
    }
    
    arbiter->move(temp);
    if(arbiter->ball_kickable()) valid_counter++;
    if(valid_counter > 30){
        valid_counter = 0;
        return 1;
    }
    
    return 0;
}

int Goto_specloc::valid_conds()
{
    // if it is opponent's kick and closer than 11 meters, can't go any nearer
    if(arbiter->opponent_special_situation() && 
       arbiter->ball.loc.get_len() < 11.0 ) return 0;
    // if close enough to specloc don't go any nearer
    if(arbiter->specloc.loc.get_len() < 0.5) return 0;
    // if time not running, can't move
    if(!arbiter->referee.time_running) return 0;
    // if specloc not valid, can't go to there
    if(!arbiter->specloc.valid) return 0;
    if(!arbiter->closest_to_ball()) return 0;
    return 1;
}


int Turnto_goal::valid_conds()
{
    if(on && on < 10) return 1;
    on = 0;

    if(arbiter->ball.loc.get_len()>Cons2::kick_max_dist()) return 0;

    if(!arbiter->opp_goal.valid){
        return 0;
    }
    
    if(fabs(arbiter->opp_goal.loc.center.get_dir()) < 0.05){
        return 0;
    }
    
    return 1;
}

int Avoid_offside_sm::valid_conds()
{
    if( arbiter->ball.loc.get_len() < 7.0 ) return 0;
    
    if( arbiter->offside.valid && arbiter->offside.loc.get_len() > 2.0 )
        return 1;
    return 0;
}

int Move_from_offside::valid_conds()
{
     int curr_time = arbiter->curr_time;
    
    if(end_time){ // if work cycle has ended, check if we can begin again
        // if can't start yet, return
        if(curr_time - end_time < MIN_GAP_BETW_CYCLES ) return 0;
        // else mark that end time is no longer the limit
        end_time = 0;
        // and start time is not the limit
        start_time = 0; 
        // return valid_conds = 1!
        return 1;
    }
    
    if(start_time){//if end time is zero and start time>zero, work cycle is on
        // check if we can still continue the cycle
        if(curr_time - start_time < MAX_WORK_CYCLE) return 1;
        // if cycle is already long enough, mark this moment as end_time
        // end return valid_conds = 0!
        end_time = curr_time;
        return 0;
    }
    // end_time == start_time == 0. return valid_conds = 1!
    return 1;
}

int Offside_lookaround::valid_conds()
{
    int curr_time = arbiter->curr_time;
    
    if(end_time){ // if work cycle has ended, check if we can begin again
        // if can't start yet, return
        if(curr_time - end_time < MIN_GAP_BETW_CYCLES ) return 0;
        // else mark that end time is no longer the limit
        end_time = 0;
        // and start time is not the limit
        start_time = 0; 
        // return valid_conds = 1!
        return 1;
    }
    
    if(start_time){//if end time is zero and start time>zero, work cycle is on
        // check if we can still continue the cycle
        if(curr_time - start_time < MAX_WORK_CYCLE) return 1;
        // if cycle is already long enough, mark this moment as end_time
        // end return valid_conds = 0!
        end_time = curr_time;
        return 0;
    }
    // end_time == start_time == 0. return valid_conds = 1!
    return 1;
    
}
