// -*- mode:C++ -*-
// File:   State_machine.h
// Author: Jussi Pajala 19.3.1998,  jussip@tks.oulu.fi
//
// this file declares class for the main state machine
// and classes to assist it.


#ifndef _STATE_MACHINE_H
#define _STATE_MACHINE_H

#include <stdlib.h>
#include "Arbiter.h"
class State_list;


class Prim_state
{
public:
    Prim_state(Arbiter *arbiterpointer);
    virtual int execute() = 0;// returns zero on success
    virtual int valid_conds() = 0;
    virtual void print_state_name();
    virtual void mark_state_on(){ state_on = 1; }
    
    virtual void reset();
protected:
    Arbiter *arbiter;
    char name[40];
    
    int state_on; // tells wheter the state was executed on previous call 
    //               or is this the first try
};



class State_machine:public Prim_state
{
public:
    State_machine(Arbiter *arbiterpointer);
    virtual ~State_machine();
    virtual int execute();// returns zero on success
    virtual int valid_conds();
    
protected:
    Prim_state *curr_state;
    State_list *state_list; // points to the first list node
};

class Main_state_machine:public State_machine
{
public:
    Main_state_machine(Arbiter *arbiterpointer);
};

class Make_close_goal:public Prim_state
{
public:
    Make_close_goal(Arbiter *arbiterpointer);
    
    virtual int execute();
    virtual int valid_conds();
};

class Make_goal:public Prim_state
{
public:
    Make_goal(Arbiter *arbiterpointer);
    
    virtual int execute();
    virtual int valid_conds();
};
class Pass_close:public Prim_state
{
public:
    Pass_close(Arbiter *arbiterpointer);
    
    virtual int execute();
    virtual int valid_conds();
};

class Pass:public Prim_state
{
public:
    Pass(Arbiter *arbiterpointer);
    
    virtual int execute();
    virtual int valid_conds();
};

class Dribble:public Prim_state
{
public:
    Dribble(Arbiter *arbiterpointer);
    
    virtual int execute();
    virtual int valid_conds();
};
class KickFar:public Prim_state
{
public:
    KickFar(Arbiter *arbiterpointer);
    
    virtual int execute();
    virtual int valid_conds();
};
class PanicKick:public Prim_state
{
public:
    PanicKick(Arbiter *arbiterpointer);
    
    virtual int execute();
    virtual int valid_conds();
protected:
    int just_executed;
};

class Lookaround:public Prim_state
{
public:
    Lookaround(Arbiter *arbiterpointer);
        
    virtual int execute();
    virtual int valid_conds();
};
class Turnto_ball:public Prim_state
{
public:
    Turnto_ball(Arbiter *arbiterpointer);
        
    virtual int execute();
    virtual int valid_conds();
};
class Goto_defloc:public Prim_state
{
public:
    Goto_defloc(Arbiter *arbiterpointer);
        
    virtual int execute();
    virtual int valid_conds();
};

class Catch_ball:public Prim_state
{
public:
    Catch_ball(Arbiter *arbiterpointer);
        
    virtual int execute();
    virtual int valid_conds();
};

class Special_situation: public State_machine
{
public:
    Special_situation(Arbiter *arbiterpointer);
    virtual int valid_conds();
};

class Goto_specloc: public Prim_state
{
public: 
    Goto_specloc(Arbiter *arbiterpointer);
    virtual int valid_conds();
    virtual int execute();
};
class Turnto_goal: public Prim_state
{
public: 
    Turnto_goal(Arbiter *arbiterpointer);
    virtual int valid_conds();
    virtual int execute();
private:
    int on;
};
class Avoid_offside_sm: public State_machine
{
public: 
    Avoid_offside_sm(Arbiter *arbiterpointer);
    virtual int valid_conds();
};

class Move_from_offside: public Prim_state
{
protected:
    int end_time;
    int start_time;
    enum { MIN_GAP_BETW_CYCLES = 2};
    enum { MAX_WORK_CYCLE = 20 };
    
public: 
    Move_from_offside(Arbiter *arbiterpointer);
    virtual int valid_conds();
    virtual int execute();
    virtual void reset(){ end_time = arbiter->curr_time; };
    
};
class Offside_lookaround: public Prim_state
{
protected:
    int end_time;
    int start_time;
    enum { MIN_GAP_BETW_CYCLES = 2 };
    enum { MAX_WORK_CYCLE = 8};
public: 
    Offside_lookaround(Arbiter *arbiterpointer);
    virtual int valid_conds();
    virtual int execute();
    virtual void reset(){end_time = arbiter->curr_time;};
};



// This class declares list for states
class State_list
{

public:
     State_list(Prim_state *newstate = (Prim_state*)NULL, 
                State_list *newnext = (State_list*)NULL);
     ~State_list();
     
     int add_state(Prim_state *newstate); // add node and state to the list 
     State_list *get_next() { return next; } // get pointer to the next node of the list
     Prim_state *get_state() { return state; } // get state pointer
     
protected:
     State_list *next;  // next node of the list
     Prim_state *state;    // state that the node is refering to
};

#endif // _STATE_MACHINE_H
