// -*- mode:C++ -*-                                     
// File:   Vectors.h
// Author: Jussi Pajala 5.5.1997,  jussip@tks.oulu.fi   
//
// This file declares Vector class
//
// This class is originally designed for RARS robot Vector 

#ifndef VECTOR_H
#define VECTOR_H

#include <math.h>
#include <fstream.h>

class Vector {
     float x;    // x length
     float y;    // y length
     float dir;  // direction
     float len;  // total length
public:
     Vector();                     // constructor
     Vector(float, float);         // constructor

     void reset()
     {
          x = 0.0;
          y = 0.0;
          dir = 0.0;
          len = 0.0;
     };
     
//     Vector(Vector); not necessary to implement, created automatically
     void set_xy(float,float);   // set new x and y
     void set_dir_len(float,float); // set new direction and length
     void lengthen(float);     // lengthen vector by a value of float
     void turn(float);        // turns Vector by (float) radians

     inline float get_x() const { return x; };
     inline float get_y() const { return y; };
     inline float get_dir()const { return dir; };
     inline float get_len() const {return len; };

     inline void normalize();
     inline float normalize(float& angle);
     
     float angle_from(const Vector& v);
     float angle_to(const Vector& v);
     
     // jpr 9.8.1997
     inline float dst(const Vector& v)
     {
          return sqrt((x-v.x)*(x-v.x) + (y-v.y)*(y-v.y));
     }
     
     inline float weighted_dst( float x_weight, const Vector& v)
     {
          return sqrt(x_weight*x_weight*(x-v.x)*(x-v.x) + (y-v.y)*(y-v.y));
     }
    // print vector information to error stream
     void err_print_xy();
     void err_print_dir_len();
    // print vector information to standard output
     void print_xy();
     void print_dir_len();
    // print vector information to file
     void fprint_xy(ofstream &file);
     void fprint_dir_len(ofstream &file);
    
     
     friend Vector operator*(float r,const Vector& v);
     friend Vector operator/(const Vector& v,float r);
     friend Vector operator+(const Vector& v,const Vector& w);
     friend Vector operator-(const Vector& v,const Vector& w);

     friend Vector operator-(const Vector& v);  // changes direction to opposite

     friend float operator*(const Vector& v,const Vector& w); // ristitulo
     friend float operator,(const Vector& v,const Vector& w); // pistetulo
     friend int operator==(const Vector&v, const Vector& w);
};

#endif
