#ifdef VIS
// file    : Visual.cc   
#include <pfstream.h>
#include "Visual.h"
#include "decls.h"
//#define DRAW_LOC
#define DRAW_EXEC_LOC    
#define DRAW_BALL
//#define DRAW_HEADING    
#define DRAW_EXEC_HEADING
//#define DRAW_POS_CALC_OBJECTS

Visual::Visual()
{
     my_id = VISUAL_ID;
     inhibited = NO;
     if(Cons2::Visual_local){
          min_x = -15.0, max_x = 15.0, min_y = -15.0, max_y = 15.0;
          scaling = 16.0;
     } else {
          min_x = -62.5, max_x = 62.5, min_y = -44.0, max_y = 44.0;
          scaling = 4.0;
     }
     empty_str[0] = '\0';
     char title[80];
     sprintf(title,"Environment state by player %d",Cons2::me_unum());
     if(Cons2::Visual_execute){
         vis = new v_producer("|bin/visualizer", min_x, max_x, min_y, max_y, scaling, title);
         cout << "VISUALIZER initialized, max_obj_id=" 
              << vis->max_dyn_obj_id () << endl;
     }
     if(Cons2::debug_level_startup()) cerr << "Visual: CONSTRUCTOR executed" << endl;
}

Visual::~Visual()
{
    if(Cons2::Visual_execute)  vis->quit ();
    if(Cons2::debug_level_exit()) cerr << "Visual: DESTRUCTOR executed" << endl;
}

void Visual::Init(EmptySamba *sambapointer)
{
    samba = sambapointer;
    ma_me = (Ma_me*)samba->get_pointer( MA_ME_ID );
    ma_ball = (Ma_ball*)samba->get_pointer( MA_BALL_ID );
    ma_teammate = (Ma_teammate*)samba->get_pointer( MA_TEAMMATE_ID );
    ma_opponent = (Ma_opponent*)samba->get_pointer( MA_OPPONENT_ID );
    ma_specloc = (Ma_special_loc*)samba->get_pointer( MA_SPECIAL_LOC_ID );
}

void Visual::req_static_objects()
{
    int i;
    float x,y;
    char namestr[10];
    namestr[0] = '\0';
    if(Cons2::Visual_local){
         // body
         vis->send_req (vis->stat_obj_id(), V_OBJ_DISC, 0.0, 0.0, 0, 
                        Cons2::player_size(),"", V_BLACK);
         // Kick radius
         vis->send_req (vis->stat_obj_id(), V_OBJ_CIRC, 0.0, 0.0, 0, 
                        Cons2::player_size()+1.0,"", V_BLACK);
         vis->send_req (vis->stat_obj_id(), V_OBJ_CIRC, 0.0, 0.0, 0, 
                        5.0,"5m", V_BLACK);
         vis->send_req (vis->stat_obj_id(), V_OBJ_CIRC, 0.0, 0.0, 0, 
                        10.0,"10m", V_BLACK);
         vis->send_req (vis->stat_obj_id(), V_OBJ_CIRC, 0.0, 0.0, 0, 
                        15.0,"15m", V_BLACK);
         vis->send_req (vis->stat_obj_id(), V_OBJ_CIRC, 0.0, 0.0, 0, 
                        20.0,"20m", V_BLACK);
    } 
    else
         for(i = 0; i < NFLAGS; i++){
              Cons2::flag_coords((FLAGTYPES)i, x, y);
              if (vis->send_req (vis->stat_obj_id(), V_OBJ_DISC, x, y, 0, 1, 
                                 namestr, V_BLACK)==-1)
                   cerr << "Visualizing error detected! line: " << __LINE__ << endl;
         }
}

FLAG Visual::valid_conds()
{
    if(Cons2::Visual_execute) return YES;
    return NO;
}

int Visual::transform()
{
    static int static_done = 0;
    int i = 0;
    Vector temp;
    if(!static_done){
        static_done = 1;
        req_static_objects();
    }
    ma_me->read_sens(me_sens);
    ma_me->read_exec(me_exec);
    

    // ADDED ********************
    if(Cons2::Visual_local){
         ma_ball->read(ball);
         sprintf(name_str,"Ball %f1.2", ball.cer);
// Nyt arveltu pallon sijainti
         vis->send_req(i++, V_OBJ_CIRC, ball.loc.get_x(), ball.loc.get_y(),
                       0, Cons2::ball_size(), name_str, V_BLACK);
         
// Nyt Katselu sektori
         float turn_angle = ma_me->read_view_angle()*0.5;
         vis->send_req(i++, V_OBJ_LINE, 0.0,0.0, -turn_angle, 5, "", V_BLUE);
         vis->send_req(i++, V_OBJ_LINE, 0.0,0.0, +turn_angle, 5, "", V_BLUE);
    } else {
         // end ADDED ********************
         
#ifdef DRAW_LOC
         sprintf (name_str,"ME");
         if (vis->send_req (i++, V_OBJ_DISC, me_sens.g_loc.get_x(), 
                            me_sens.g_loc.get_y(), 0, 1, name_str, V_BLUE)==-1)
              cerr << "Visualizing error detected!" << endl;
#endif

#ifdef DRAW_EXEC_LOC    
    if(Cons2::Visual_exec_loc){
          name_str[0] = 0;
          vis->send_req(i++, V_OBJ_CIRC, me_exec.g_loc.get_x(), 
                        me_exec.g_loc.get_y(), 0, 1, name_str, V_RED);
     }
#endif
#ifdef DRAW_BALL
    if(Cons2::Visual_ball){
        // draw ball
        ma_ball->read(ball);
        sprintf(name_str,"Ball");
        temp = loc2g_loc(ball.loc, me_exec.g_loc, me_exec.heading);
                
        vis->send_req(i++, V_OBJ_CIRC, ball.g_loc./*temp.*/get_x(), ball.g_loc./*temp.*/get_y(),
                       0, 1, name_str, V_BLACK);
    }
#endif
#ifdef DRAW_HEADING
    if(Cons2::Visual_heading){
           name_str[0] = 0;
           vis->send_req(i++, V_OBJ_LINE, me_sens.g_loc.get_x(), 
                     me_sens.g_loc.get_y(), me_sens.heading, 5,
                      name_str, V_BLUE);
    }
#endif
#ifdef DRAW_EXEC_HEADING
    if(Cons2::Visual_exec_heading){
        name_str[0] = 0;
        vis->send_req(i++, V_OBJ_LINE, me_exec.g_loc.get_x(), 
                      me_exec.g_loc.get_y(), me_exec.heading, 5,
                      name_str, V_RED);
    }
#endif
#ifdef DRAW_POS_CALC_OBJECTS
    if(Cons2::Visual_pos_calc_objects){
        float x1,y1,x2,y2;
        ma_me->read_pos_calc_objects(x1, y1, x2, y2);
        
        name_str[0] = '1';
        name_str[1] = 0;
        vis->send_req(i++, V_OBJ_DISC, x1, y1, 0 , 1,
                      name_str, V_RED);
        name_str[0] = '2';
        vis->send_req(i++, V_OBJ_DISC, x2, y2, 0 , 1,
                      name_str, V_RED);
    }
#endif
    int k; 
    int t;

     if(Cons2::Visual_teammates){
         ma_teammate->read(plrs);
         for(k = 0; k < 10 && k < Cons2::Visual_teammates; k++){
             t = plrs.order[k];
             if(plrs.player[t].cer < 0.05) plrs.player[t].g_loc.set_xy(-100.0,0.0);
             sprintf(name_str,"%d:%d",k,t+1);
             vis->send_req(k+i, V_OBJ_DISC, plrs.player[t].g_loc.get_x(),
                           plrs.player[t].g_loc.get_y(), 
                           0, 1, name_str, V_BLUE);
         }
         i = k+i;
         }
     if(Cons2::Visual_opponents){
         ma_opponent->read(plrs);
         for(k = 0; k < 10 && k < Cons2::Visual_opponents; k++){
             t = plrs.order[k];
             if(plrs.player[t].cer < 0.05) plrs.player[t].g_loc.set_xy(-100.0,0.0);
             sprintf(name_str,"%d:%d",k,t+1);
             vis->send_req(k+i, V_OBJ_DISC, plrs.player[t].g_loc.get_x(),
                           plrs.player[t].g_loc.get_y(), 
                           0, 1, name_str, V_RED);
         }
         i = k+i;
     }
     if(Cons2::Visual_special_loc){
         static Special_loc specloc;
         ma_specloc->read(specloc);
         if(specloc.valid){
             Vector special_loc;
             strcpy(name_str,"spec_loc");
             special_loc = loc2g_loc(specloc.loc, me_exec.g_loc, 
                                     me_exec.heading);
             vis->send_req(i++, V_OBJ_DISC, special_loc.get_x(),
                           special_loc.get_y() , 0, 1, name_str, V_BLUE);
         }
     }
/*
    if(Cons2::Visual_opp_goal){
          // opponent goal
          opp_goal_in->signal.read(goal_loc, goal_angle, goal_cer, goal_g_loc, goal_width, goal_g_cer, goal_time, goal_id);
          goal_loc.turn(me_exec_heading);
          goal_loc = goal_loc + me_exec_loc;
          goal_angle += me_exec_heading;
          my_vis.put_req(i++, V_OBJ_LSEG, goal_loc.get_x(), goal_loc.get_y(),
                         goal_angle, goal_width,"opp_goal", V_RED);
     }
     if(Cons2::Visual_own_goal){
          // own goal
          own_goal_in->signal.read(goal_loc, goal_angle, goal_cer, goal_g_loc, goal_width, goal_g_cer, goal_time, goal_id);
          goal_loc.turn(me_exec_heading);
          goal_loc = goal_loc + me_exec_loc;
          goal_angle += me_exec_heading;
          my_vis.put_req(i++, V_OBJ_LSEG, goal_loc.get_x(), goal_loc.get_y(),
                         goal_angle, goal_width,"own_goal", V_RED);
     }
     if(Cons2::Visual_goal_flags){
          // calculations for goal flags
          // calculate global location
          oppg_flags_in->signal.read(flag_prt, flag_prc, flag_prb, flag_time, flag_id);
          owng_flags_in->signal.read(flag_plt, flag_plc, flag_plb, flag_time, flag_id);

          flag_prt.loc.turn(me_exec_heading);
          flag_prt.loc = flag_prt.loc + me_exec_loc;
     
          flag_prc.loc.turn(me_exec_heading);
          flag_prc.loc = flag_prc.loc + me_exec_loc;
     
          flag_prb.loc.turn(me_exec_heading);
          flag_prb.loc = flag_prb.loc + me_exec_loc;
     
          flag_plt.loc.turn(me_exec_heading);
          flag_plt.loc = flag_plt.loc + me_exec_loc;
     
          flag_plc.loc.turn(me_exec_heading);
          flag_plc.loc = flag_plc.loc + me_exec_loc;
          
          flag_plb.loc.turn(me_exec_heading);
          flag_plb.loc = flag_plb.loc + me_exec_loc;
          
          my_vis.put_req(i++, V_OBJ_DISC, flag_plt.loc.get_x(), flag_plt.loc.get_y(),
                         0, 1,"plt", V_BLACK);
          my_vis.put_req(i++, V_OBJ_DISC, flag_plc.loc.get_x(), flag_plc.loc.get_y(),
                         0, 1,"plc", V_BLACK);
          my_vis.put_req(i++, V_OBJ_DISC, flag_plb.loc.get_x(), flag_plb.loc.get_y(),
                         0, 1,"plb", V_BLACK);
          
          my_vis.put_req(i++, V_OBJ_DISC, flag_prt.loc.get_x(), flag_prt.loc.get_y(),
                         0, 1,"prt", V_BLACK);
          my_vis.put_req(i++, V_OBJ_DISC, flag_prc.loc.get_x(), flag_prc.loc.get_y(),
                         0, 1,"prc", V_BLACK);
          my_vis.put_req(i++, V_OBJ_DISC, flag_prb.loc.get_x(), flag_prb.loc.get_y(),
                         0, 1,"prb", V_BLACK);
          
     }
     if(Cons2::Visual_border_flags){
          //     calculations for border flags
          //     calculate global location     
          brd_flags_in ->signal.read(flag_lt, flag_ct, flag_rt, flag_lb,
                                     flag_cb, flag_rb,flag_time, flag_id);
          
          flag_lt.loc.turn(me_exec_heading);
          flag_lt.loc = flag_lt.loc + me_exec_loc;
          
          flag_ct.loc.turn(me_exec_heading);
          flag_ct.loc = flag_ct.loc + me_exec_loc;
          
          flag_rt.loc.turn(me_exec_heading);
          flag_rt.loc = flag_rt.loc + me_exec_loc;
          
          flag_lb.loc.turn(me_exec_heading);
          flag_lb.loc = flag_lb.loc + me_exec_loc;
          
          flag_cb.loc.turn(me_exec_heading);
          flag_cb.loc = flag_cb.loc + me_exec_loc;
          
          flag_rb.loc.turn(me_exec_heading);
          flag_rb.loc = flag_rb.loc + me_exec_loc;
          
          my_vis.put_req(i++, V_OBJ_DISC, flag_lt.loc.get_x(), flag_lt.loc.get_y(),
                         0, 1,"lt", V_BLACK);
          my_vis.put_req(i++, V_OBJ_DISC, flag_ct.loc.get_x(), flag_ct.loc.get_y(),
                         0, 1,"ct", V_BLACK);
          my_vis.put_req(i++, V_OBJ_DISC, flag_rt.loc.get_x(), flag_rt.loc.get_y(),
                         0, 1,"rt", V_BLACK);
          
          my_vis.put_req(i++, V_OBJ_DISC, flag_lb.loc.get_x(), flag_lb.loc.get_y(),
                         0, 1,"lb", V_BLACK);
          my_vis.put_req(i++, V_OBJ_DISC, flag_cb.loc.get_x(), flag_cb.loc.get_y(),
                         0, 1,"cb", V_BLACK);
          my_vis.put_req(i++, V_OBJ_DISC, flag_rb.loc.get_x(), flag_rb.loc.get_y(),
                         0, 1,"rb", V_BLACK);
          
          
     }
     if(Cons2::Visual_lines){
          // closest points of all the lines
          // calculate global location and orientation 
          border_msig_in->signal.read(line_top, line_bottom, line_l_t, line_l_b,
                                      line_r_t, line_r_b, line_time, line_id);
          
          line_top.loc.turn(me_exec_heading);
          line_top.loc = line_top.loc + me_exec_loc;
          line_top.angle -= me_exec_heading;
          
          line_bottom.loc.turn(me_exec_heading);
          line_bottom.loc = line_bottom.loc + me_exec_loc;
          line_bottom.angle -= me_exec_heading;
          
          line_r_t.loc.turn(me_exec_heading);
          line_r_t.loc = line_r_t.loc + me_exec_loc;
          line_r_t.angle -= me_exec_heading;
          
          line_r_b.loc.turn(me_exec_heading);
          line_r_b.loc = line_r_b.loc + me_exec_loc;
          line_r_b.angle -= me_exec_heading;
          
          line_l_t.loc.turn(me_exec_heading);
          line_l_t.loc = line_l_t.loc + me_exec_loc;
          line_l_t.angle -= me_exec_heading;
          
          line_l_b.loc.turn(me_exec_heading);
          line_l_b.loc = line_l_b.loc + me_exec_loc;
          line_l_b.angle -= me_exec_heading;
          
          my_vis.put_req(i++, V_OBJ_LSEG, line_top.loc.get_x(), line_top.loc.get_y(),
                         line_top.angle, 7,"line_top", V_BLACK);
          my_vis.put_req(i++, V_OBJ_LSEG, line_bottom.loc.get_x(), line_bottom.loc.get_y(),
                         line_bottom.angle,7,"line_bottom", V_BLACK);
          my_vis.put_req(i++, V_OBJ_LSEG, line_l_t.loc.get_x(), line_l_t.loc.get_y(),
                    line_l_t.angle,7,"line_l_t", V_BLACK);
          my_vis.put_req(i++, V_OBJ_LSEG, line_l_b.loc.get_x(), line_l_b.loc.get_y(),
                    line_l_b.angle,7,"line_l_b", V_BLACK);
          my_vis.put_req(i++, V_OBJ_LSEG, line_r_b.loc.get_x(), line_r_b.loc.get_y(),
                         line_r_b.angle,7,"line_r_b", V_BLACK);
          my_vis.put_req(i++, V_OBJ_LSEG, line_r_t.loc.get_x(), line_r_t.loc.get_y(),
                         line_r_t.angle,7,"line_r_t", V_BLACK);
     }
     */

/*
     if(Cons2::Visual_defloc){
          defloc_msig_in->signal.read( defloc_g_loc, defloc_loc,
                                       defloc_time, defloc_id );
          strcpy(name_str,"defloc");
          
          my_vis.put_req(i++, V_OBJ_DISC, defloc_g_loc.get_x(),
                         defloc_g_loc.get_y() , 0, 1, name_str, V_RED);
     }*/
     }
    return 0;
    
}
/*
     for (int cnt = 0; cnt < 5000; cnt++) 
          {
               sprintf (name_str,"Obj%d",0);
               if (my_producer.send_req (0, V_OBJ_CIRC, x0, y0, 0, 2, name_str, V_RED) == -1)
                    cerr << "Visualizing error detected!" << endl;
               increment (min_x, max_x, x0, x0_step);
               increment (min_y, max_y, y0, y0_step);

               sprintf (name_str,"Obj %d",1);
               my_producer.send_req (1, V_OBJ_LSEG, 0, 0, dir1, 10, name_str, V_BLUE);
               set_dir (dir1_step, dir1);

               my_producer.send_req (2, V_OBJ_DISC, x2, y2, 0, 1, empty_str, V_GREEN);
               increment (min_x, max_x, x2, x2_step);
               increment (min_y, max_y, y2, y2_step);

               sprintf (name_str,"Obj%d",3);
               my_producer.send_req (3, V_OBJ_LSEG, 20,-10, dir3, 5, name_str, V_PINK);
               set_dir (dir3_step, dir3);

               sprintf (name_str,"Obj%d",4);
               my_producer.send_req (4, V_OBJ_DISC, x4, y4, 0, 1.2, name_str, V_VIOLET);
               increment (min_x, max_x, x4, x4_step);
               increment (min_y, max_y, y4, y4_step);
          }
     my_producer.quit ();
}
*/

#endif //VIS
