// file    : Vs_referee.cc   

#include "decls.h"
//#include "Soccer.h"
#include <Vs/Vs_referee.h>
#include "Cons2.h"
Vs_referee::Vs_referee()
{
    my_id = VS_REFEREE_ID;
    inhibited = NO;

    if(Cons2::debug_level_startup()){
        if( Cons2::side_char() == 0 )
            cerr << "Vs_referee: side char not yet initialized!" << endl;
        else cerr << endl << "Side char initialized" << endl << endl;
    }
    
    referee_state.team_goals = referee_state.opp_goals = 0;
    referee_state.game_half = first_half;
    referee_state.game_state = before_kick_off;
    referee_state.game_state_changed = 1;
    
    if(Cons2::debug_level_startup()) 
        cerr << "Vs_referee: CONSTRUCTOR executed" << endl;

}

Vs_referee::~Vs_referee()
{
     if(Cons2::debug_level_exit()) 
         cerr << "Vs_referee: DESTRUCTOR executed" << endl;
}

void Vs_referee::Init(EmptySamba *sambapointer)
{
    samba = sambapointer;
    sensor = (Sensor*)samba->get_pointer( SENSOR_ID );
}

void Vs_referee::analyze_message(Audioinfo& message)
{

     // if referee has said something, game state is changed
     // when game_state_changed == 1, all modules should
     // reset their filters!!
     referee_state.game_state_changed = 1;
     // PLAY ON
     if(!strcmp(message.message,"play_on"))
          referee_state.game_state = play_on;
     // GOAL_R_
     else if(!strncmp(message.message,"goal_r_",7)){
          int goals,length;
          referee_state.game_state = after_goal;
          sscanf(&(message.message[7]),"%d %n", &goals, &length);
          if(Cons2::side_char() == 'r') 
               referee_state.team_goals = goals;
          else referee_state.opp_goals = goals;
     }
     // GOAL_L_
     else if(!strncmp(message.message,"goal_l_",7)){
          int goals,length;
          referee_state.game_state = after_goal;
          sscanf(&(message.message[7]),"%d %n", &goals, &length);
          if(Cons2::side_char() == 'l') 
               referee_state.team_goals = goals;
          else referee_state.opp_goals = goals;
     }
     // KICK_OFF
     else if(!strncmp(message.message,"kick_off_",9)){
          if(message.message[9] == Cons2::side_char())
               referee_state.game_state = team_kick_off;
          else referee_state.game_state = opp_kick_off;
     }
     // KICK_IN
     else if(!strncmp(message.message,"kick_in_",8)){
          if(message.message[8] == Cons2::side_char())
               referee_state.game_state = team_kick_in;
          else referee_state.game_state = opp_kick_in;
     }
     // CORNER KICK
     else if(!strncmp(message.message,"corner_kick_",12)){
          if(message.message[12] == Cons2::side_char())
               referee_state.game_state = team_corner_kick;
          else referee_state.game_state = opp_corner_kick;
     }
     // GOAL_KICK
     else if(!strncmp(message.message,"goal_kick_",10)){
          if(message.message[10] == Cons2::side_char())
               referee_state.game_state = team_goal_kick;
          else referee_state.game_state = opp_goal_kick;
     }
     // FREE KICK
     else if(!strncmp(message.message,"free_kick_",10)){
          if(message.message[10] == Cons2::side_char())
               referee_state.game_state = team_free_kick;
          else referee_state.game_state = opp_free_kick;
     }
     // OFFSIDE KICK
     else if(!strncmp(message.message,"offside_",8)){
          if(message.message[8] == Cons2::side_char())
              referee_state.game_state = opp_offside;
          else referee_state.game_state = team_offside;
     }
     // FREE KICK BECAUS GOALIE HAS CATCHED THE BALL
     else if(!strncmp(message.message,"goalie_catch_ball_",18)){
          if(message.message[18] == Cons2::side_char())
              referee_state.game_state = team_goalie_catch;
          else referee_state.game_state = opp_goalie_catch;
     }
     else if(!strncmp(message.message,"drop_ball",9)){
         referee_state.game_state = play_on;
     }
     
     // FOUL
     else if(!strncmp(message.message,"foul_",5)){
          if(message.message[5] == Cons2::side_char()){
               // some player of our team has done something wrong!
               // check if it's me!
//               int unum;
//               sscanf(&(message.message[7]),"%d",&unum);
//               if(unum == me_unum) do something!
               referee_state.game_state = play_on;
          }
          else referee_state.game_state = play_on;
     }
     // BEFORE KICK OFF
     else if(!strcmp(message.message,"before_kick_off")){
          referee_state.game_state = before_kick_off;
     }
     // TIME OVER
     else if(!strcmp(message.message, "time_over")
          || !strcmp(message.message, "time_up") ){
          referee_state.game_state = time_over;
#ifndef NOPRINTS
          cout << "The game is over." << endl;
          if(referee_state.team_goals >
             referee_state.opp_goals)
               cout << "WE WON!!!!" << endl;
          else cout << "Better team won!" << endl;
#endif
          exit(0);
     }
     // SECOND HALF STARTS
     else if(!strcmp(message.message,"half_time")){
          referee_state.game_half = second_half;
     }
     // THIRD HALF STARTS
     else if(!strcmp(message.message,"time_extended")){
          referee_state.game_half = time_extended;
     }
    
     else {
         cerr << "Referee message not understood!!!" << endl;
         cerr << "Message: " << message.message << endl;
         cerr << __FILE__ << " " << __LINE__ << endl;
     }
     
     
/*     cerr << "Referee has declared game stage to be:";
     
     switch(referee_state.game_state){
     case play_on:          cerr << " play on"; break;
     case opp_kick_off:     cerr << " opp_kick_off"; break;
     case team_kick_off:    cerr << " team_kick_off"; break;
     case opp_kick_in:      cerr << " opp_kick_in"; break;
     case team_kick_in:     cerr << " team_kick_in"; break;
     case opp_corner_kick:  cerr << " opp_corner_kick:"; break;
     case team_corner_kick: cerr << " team_corner_kick:"; break;
     case opp_goal_kick:    cerr << " opp_goal_kick:"; break;
     case team_goal_kick:   cerr << " team_goal_kick:"; break;
     case opp_free_kick:    cerr << " opp_free_kick:"; break;
     case team_free_kick:   cerr << " team_free_kick:"; break;
     case before_kick_off:  cerr << " before_kick_off:"; break;
     case after_goal:       cerr << " after_goal: " << endl
                                 << "Game is now: ("
                                 << referee_state.team_goals << "-"
                                 << referee_state.opp_goals
                                 << ") (team-opp)" << endl; break;
     default: {
              cerr << " unknown" << endl
                   << message.message << endl
                   << __FILE__ << " " << __LINE__ << endl;
              exit(1);
              }
     }
     cerr << endl;
*/   
}

int Vs_referee::game_is_over()
{
     if(referee_state.game_state == time_over)
         return 1; // game is over
     
     return 0; // game continues
}

void Vs_referee::invalidate_all()
{
    referee_state.valid = 0;
}


void Vs_referee::read(Referee_vs &to)
{
    check_executed();
    to = referee_state;
}

int Vs_referee::transform()
{
    int i;
    referee_state.game_state_changed = 0;
     
    if( !Cons2::side_char() ){
        cerr << " This thing doesn't work! " << __FILE__ << " "
             << __LINE__ << endl;
        exit(1);
    }
    sensor->read_audio_sensor(messages);

    // search for messages sent by referee 
    for( i = 0; i < MAX_HEARD_MSG; i++)
        if( messages[i].valid && messages[i].talker == TALKER_REFEREE )
             // analyze referee's message and change
            // referee_state variable's state
            {
                //cerr << "Heard something..." << endl;
                analyze_message( messages[i] );
            }
    referee_state.time_running = 1;
        
    if(referee_state.game_state == after_goal ||
       referee_state.game_state == before_kick_off )
        referee_state.time_running = 0;
    
    if(referee_state.game_state == before_kick_off
       && sensor->is_any_valid()
       && ( (curr_time > 15 && curr_time < 2999 ) || 
            (curr_time > 3015 && curr_time < 5999 ) || 
            (curr_time > 6012)))
        {
#ifndef NOPRINTS
            cout << "Changing game state without referee notification..." 
                 << endl;
#endif
            referee_state.game_state = play_on;
            referee_state.game_state_changed = 1;
            referee_state.time_running = 1;
        }
    if(referee_state.game_state == after_goal ||
       referee_state.game_state == before_kick_off)
        referee_state.time_running = 0;
    
    referee_state.valid = 1;
    return 0;
}
