#include "Const.h"
#include "Cons2.h"
#include "cfg.h"
#include "Environment.h"
#include <iostream.h>
#include <math.h>
Environment::Environment(int exp_no)
{
     PLAYER_RAD = Cons2::player_size();
     BALL_RAD = Cons2::ball_size();
     GOAL_WIDTH = Cons2::goal_width();
     read_config_file(exp_no);

     // Visualisointi:
     char *title = "Testing Actionmaps";
/* VIS */
     vis = new v_producer("|/home/jpr/robocup98/samba/bin/visualizer",
                          -CS_MAX_VAL, CS_MAX_VAL, -CS_MAX_VAL, CS_MAX_VAL, CS2PIX, title);
     max_dyn_obj_id = vis->max_dyn_obj_id();
     stat_obj_id = vis->stat_obj_id();
/* SIV */

/*
     cout << "- VISUALIZER initialized, max_dyn_obj_id=" << max_dyn_obj_id 
          << " stat_obj_id=" << stat_obj_id << endl;
     cout << "  cs max value: " << CS_MAX_VAL
          << "  scaling: " << CS2PIX << endl;
*/     
/* VIS */
     // DRAW AS STATIC OBJECT:
     vis->send_req(stat_obj_id, V_OBJ_DISC, 0, 0, 0, PLAYER_RAD, "ME", V_BLUE);
     vis->send_req(stat_obj_id, V_OBJ_LSEG, 0, 0, 0,100,"", V_BLUE2);
      vis->send_req(stat_obj_id, V_OBJ_LSEG, 0, 0,0.5*M_PI ,100,"", V_BLUE2);
      vis->send_req(stat_obj_id, V_OBJ_CIRC, 0, 0,0, PLAYER_RAD + 1.0, "", V_BLUE);
/* SIV */
}

Environment::~Environment()
{
/* VIS */
     delete vis;
     vis->quit();
     vis = NULL;
/* SIV */
}

int Environment::update_environment(Vector new_vel, Vector kick, int forward)
{
     my_vel = new_vel;
     Vector ball_vel_change;
     ball_vel_change.set_dir_len (kick.get_dir(), 
                                  kick.get_len() * Cons2:: kick_power_rate());

     if(param_ball_loc.get_len() < 2.0)
          param_ball_vel = param_ball_vel + ball_vel_change;

     param_ball_vel       = Cons2::ball_decay()*param_ball_vel;
     param_ball_start_loc = param_ball_start_loc - my_vel;
     param_ball_prev_loc  = param_ball_loc - my_vel;
     param_ball_loc       = param_ball_loc + param_ball_vel - my_vel;
     param_team1_loc      = param_team1_loc + param_team1_vel - my_vel;
     param_team2_loc      = param_team2_loc + param_team2_vel - my_vel;
     param_oppo1_loc      = param_oppo1_loc + param_oppo1_vel - my_vel;
     param_oppo2_loc      = param_oppo2_loc + param_oppo2_vel - my_vel;
     param_defloc         = param_defloc - my_vel;
     param_specloc        = param_specloc - my_vel;
     param_goal_loc       = param_goal_loc - my_vel;
     param_left_pole_loc  = param_left_pole_loc - my_vel;


     // [jpr 980610] all the data is represented in the local coordinate system of the player!
     if (my_vel.get_len() > 0.001) {
          // cerr << "*** ROTATING BY " << -my_vel.get_dir() << " RAD ***" << endl;
          param_ball_vel.turn(-my_vel.get_dir());
          param_ball_start_loc.turn(-my_vel.get_dir());
          param_ball_prev_loc.turn(-my_vel.get_dir());
          param_ball_loc.turn(-my_vel.get_dir());
          param_team1_loc.turn(-my_vel.get_dir());
          param_team1_vel.turn(-my_vel.get_dir());
          param_team2_loc.turn(-my_vel.get_dir());
          param_team2_vel.turn(-my_vel.get_dir());
          param_oppo1_loc.turn(-my_vel.get_dir());
          param_oppo1_vel.turn(-my_vel.get_dir());
          param_oppo2_loc.turn(-my_vel.get_dir());
          param_oppo2_vel.turn(-my_vel.get_dir());
          param_defloc.turn(-my_vel.get_dir());
          param_specloc.turn(-my_vel.get_dir());
          param_goal_loc.turn(-my_vel.get_dir());
          param_left_pole_loc.turn(-my_vel.get_dir());
     }
     
     if(param_printvalues){
          cerr<<"ball_loc "; param_ball_loc.err_print_dir_len(); cerr<<endl;
          cerr<<"ball_vel "; param_ball_vel.err_print_dir_len(); cerr<<endl;
          cerr<<"team1_loc "; param_team1_loc.err_print_dir_len(); cerr<<endl;
          cerr<<"team2_loc "; param_team2_loc.err_print_dir_len(); cerr<<endl;
          cerr<<"oppo1_loc "; param_oppo1_loc.err_print_dir_len(); cerr<<endl;
          cerr<<"oppo2_loc "; param_oppo2_loc.err_print_dir_len(); cerr<<endl;
          cerr<<"max_player_coll_time = "<<param_max_player_coll_time<<endl;
          cerr<<"goal_loc "; param_goal_loc.err_print_dir_len(); cerr<<endl;
          cerr<<"my_vel "; my_vel.err_print_dir_len(); cerr<<endl;
     }

/* VIS */
     visualize();          
/* SIV */
     return 1;
}
int Environment::visualize()
{
     for(int cnt = FIRST_FREE_ID; cnt < pnt_id; cnt++)
          vis->send_req(cnt, V_OBJ_DISC, 0, 0, 0, PLAYER_RAD, "", V_BLACK);
     pnt_id = FIRST_FREE_ID;
     Vector temp;
     
     // For teammates and opponents draw:
     // player, kickradius, velocity
     vis->send_req(pnt_id++, V_OBJ_DISC, param_team1_loc.get_x(), param_team1_loc.get_y(),
                  0, PLAYER_RAD, "1", V_BLUE);
     vis->send_req(pnt_id++, V_OBJ_CIRC,param_team1_loc.get_x(), param_team1_loc.get_y(), 0,
                  PLAYER_RAD + 1.0, "", V_BLUE);
     vis->send_req(pnt_id++, V_OBJ_THICK_LINE,param_team1_loc.get_x(), param_team1_loc.get_y(),
                   param_team1_vel.get_dir(), param_team1_vel.get_len(),"", V_BLUE);

     vis->send_req(pnt_id++, V_OBJ_DISC, param_team2_loc.get_x(), param_team2_loc.get_y(),
                  0, PLAYER_RAD, "2", V_BLUE);
     vis->send_req(pnt_id++, V_OBJ_CIRC,param_team2_loc.get_x(), param_team2_loc.get_y(), 0,
                  PLAYER_RAD + 1.0, "", V_BLUE);
     vis->send_req(pnt_id++, V_OBJ_THICK_LINE,param_team2_loc.get_x(), param_team2_loc.get_y(),
                   param_team2_vel.get_dir(), param_team2_vel.get_len(),"", V_BLUE);
 
     vis->send_req(pnt_id++, V_OBJ_DISC, param_oppo1_loc.get_x(), param_oppo1_loc.get_y(),
                  0, PLAYER_RAD, "1", V_GREEN);
     vis->send_req(pnt_id++, V_OBJ_CIRC,param_oppo1_loc.get_x(), param_oppo1_loc.get_y(), 0,
                  PLAYER_RAD + 1.0, "", V_GREEN);   
     vis->send_req(pnt_id++, V_OBJ_THICK_LINE,param_oppo1_loc.get_x(), param_oppo1_loc.get_y(),
                   param_oppo1_vel.get_dir(), param_oppo1_vel.get_len(),"", V_GREEN);

     vis->send_req(pnt_id++, V_OBJ_DISC, param_oppo2_loc.get_x(), param_oppo2_loc.get_y(),
                  0, PLAYER_RAD, "2", V_GREEN);
     vis->send_req(pnt_id++, V_OBJ_CIRC,param_oppo2_loc.get_x(), param_oppo2_loc.get_y(), 0,
                  PLAYER_RAD + 1.0, "", V_GREEN);
     vis->send_req(pnt_id++, V_OBJ_THICK_LINE,param_oppo2_loc.get_x(), param_oppo2_loc.get_y(),
                   param_oppo2_vel.get_dir(), param_oppo2_vel.get_len(),"", V_GREEN);

     // Ball and ball's track
     vis->send_req(pnt_id++, V_OBJ_DISC, param_ball_loc.get_x(), param_ball_loc.get_y(),
                  0, BALL_RAD, "", V_RED);
     temp = param_ball_loc - param_ball_start_loc;
     vis->send_req(pnt_id++, V_OBJ_LINE,param_ball_start_loc.get_x(),
                        param_ball_start_loc.get_y(),temp.get_dir(),temp.get_len(),"", V_RED);
     vis->send_req(pnt_id++, V_OBJ_LINE,param_ball_loc.get_x(),
                        param_ball_loc.get_y(),param_ball_vel.get_dir(),param_ball_vel.get_len(),"", V_PINK);
     
     // My new calculated velocity
     vis->send_req(pnt_id++, V_OBJ_THICK_LINE, 0, 0, my_vel.get_dir(), my_vel.get_len(),"", V_BLUE);
     
     // Goal
     vis->send_req(pnt_id++, V_OBJ_THICK_LINE, param_left_pole_loc.get_x(),
                   param_left_pole_loc.get_y(),
                   goal.get_dir(),goal.get_len(),"Goal", V_BLUE);

     // defloc & specloc
     vis->send_req(pnt_id++, V_OBJ_CIRC,param_defloc.get_x(), param_defloc.get_y(), 0,
                  1.0,"D", V_PINK);
     vis->send_req(pnt_id++, V_OBJ_CIRC,param_specloc.get_x(), param_specloc.get_y(), 0,
                  1.0,"S", V_PINK);
     return 1;
}

int Environment::read_config_file(int cfg_no)
{
     FILE *cfg_fp;
     char name[200];
     sprintf(name,"Environment%d.cfg",cfg_no);
     cfg_fp = cfg_open (name); 
     
     if (cfg_fp != NULL) {
          cfg_read (cfg_fp);
          cfg_close (cfg_fp);
     } else {
          cerr << "Could not read configuration file Environment"<<cfg_no<<".cfg!" << endl;
          return 0;
     }
     float temp1, temp2;
     
     cfg_get ("testmove", &temp1); param_testmove = (int)temp1;
     cfg_get ("testkick", &temp1); param_testkick =(int)temp1;
     cfg_get ("printvalues", &temp1); param_printvalues =(int)temp1;
     cfg_get ("endtime", &temp1); param_endtime =(int)temp1;

     cfg_get ("ball_loc_x", &temp1);
     cfg_get ("ball_loc_y", &temp2);  param_ball_loc.set_xy(temp1,temp2);
     cfg_get ("ball_vel_dir", &temp1);
     cfg_get ("ball_vel_spd", &temp2); param_ball_vel.set_dir_len(temp1,temp2);

     param_ball_start_loc = param_ball_loc;

     cfg_get ("team1_loc_x", &temp1);
     cfg_get ("team1_loc_y", &temp2); param_team1_loc.set_xy(temp1,temp2);
     cfg_get ("team1_vel_dir", &temp1);
     cfg_get ("team1_vel_spd", &temp2); param_team1_vel.set_dir_len(temp1,temp2);

     cfg_get ("team2_loc_x", &temp1);
     cfg_get ("team2_loc_y", &temp2); param_team2_loc.set_xy(temp1,temp2);
     cfg_get ("team2_vel_dir", &temp1);
     cfg_get ("team2_vel_spd", &temp2); param_team2_vel.set_dir_len(temp1,temp2);

     cfg_get ("oppo1_loc_x", &temp1);
     cfg_get ("oppo1_loc_y", &temp2); param_oppo1_loc.set_xy(temp1,temp2);
     cfg_get ("oppo1_vel_dir", &temp1);
     cfg_get ("oppo1_vel_spd", &temp2); param_oppo1_vel.set_dir_len(temp1,temp2);

     cfg_get ("oppo2_loc_x", &temp1);
     cfg_get ("oppo2_loc_y", &temp2); param_oppo2_loc.set_xy(temp1,temp2);
     cfg_get ("oppo2_vel_dir", &temp1);
     cfg_get ("oppo2_vel_spd", &temp2); param_oppo2_vel.set_dir_len(temp1,temp2);

     cfg_get ("max_player_coll_time", &temp2); param_max_player_coll_time = temp2;

     cfg_get ("defloc_x", &temp1);
     cfg_get ("defloc_y", &temp2); param_defloc.set_xy(temp1,temp2);
     cfg_get ("specloc_x", &temp1);
     cfg_get ("specloc_y", &temp2); param_specloc.set_xy(temp1,temp2);

     cfg_get ("goal_loc_x", &temp1);
     cfg_get ("goal_loc_y", &temp2); param_goal_loc.set_xy(temp1,temp2);

     cfg_get ("goal_angle", &temp1); param_goal_angle = temp1;

     cfg_get ("ball_viewmap", &temp1); param_viewmap[BALL] = (int) temp1;
     cfg_get ("teammate_viewmap", &temp1); param_viewmap[TEAMMATE] = (int) temp1;
     cfg_get ("no_teammate_viewmap", &temp1); param_viewmap[NO_TEAMMATE] = (int) temp1;
     cfg_get ("opponent_viewmap", &temp1); param_viewmap[OPPONENT] = (int) temp1;
     cfg_get ("no_opponent_viewmap", &temp1); param_viewmap[NO_OPPONENT] = (int) temp1;
     cfg_get ("goal_viewmap", &temp1); param_viewmap[GOAL] = (int) temp1;
     cfg_get ("defloc_viewmap", &temp1); param_viewmap[DEFLOC] = (int) temp1;
     cfg_get ("near_viewmap", &temp1); param_viewmap[NEAR] = (int) temp1;
     cfg_get ("far_viewmap", &temp1); param_viewmap[FAR] = (int) temp1;
     cfg_get ("badsector_viewmap", &temp1); param_viewmap[BADSECTOR] = (int) temp1;

     cfg_get ("moveto_ball_viewmap", &temp1); param_viewmap[MOVETO_BALL] = (int) temp1;
     cfg_get ("moveto_player_viewmap", &temp1); param_viewmap[MOVETO_PLAYER] = (int) temp1;
     cfg_get ("moveto_defloc_viewmap", &temp1); param_viewmap[MOVETO_DEFLOC] = (int) temp1;
     cfg_get ("moveto_specloc_viewmap", &temp1); param_viewmap[MOVETO_SPECLOC] = (int) temp1;

     cfg_get ("makegoal_viewmap", &temp1); param_viewmap[MAKE_GOAL] = (int) temp1;
     cfg_get ("pass_viewmap", &temp1); param_viewmap[PASS] = (int) temp1;
     cfg_get ("dribble_viewmap", &temp1); param_viewmap[DRIBBLE] = (int) temp1;
     cfg_get ("kickfar_viewmap", &temp1); param_viewmap[KICKFAR] = (int) temp1;

     cfg_get ("moveto_ball_calculate", &temp1); param_calculate[MOVETO_BALL] = (int) temp1;
     cfg_get ("moveto_player_calculate", &temp1); param_calculate[MOVETO_PLAYER] = (int) temp1;
     cfg_get ("moveto_defloc_calculate", &temp1); param_calculate[MOVETO_DEFLOC] = (int) temp1;
     cfg_get ("moveto_specloc_calculate", &temp1); param_calculate[MOVETO_SPECLOC] = (int) temp1;
     cfg_get ("makegoal_calculate", &temp1); param_calculate[MAKE_GOAL] = (int) temp1;
     cfg_get ("pass_calculate", &temp1);     param_calculate[PASS] = (int) temp1;
     cfg_get ("dribble_calculate", &temp1);  param_calculate[DRIBBLE] = (int) temp1;
     cfg_get ("kickfar_calculate", &temp1);  param_calculate[KICKFAR] = (int) temp1;
     

     // Goal cslculations c2l = center to left pole 
     Vector goal_c2l;  goal_c2l.set_dir_len(param_goal_loc.get_dir(),Cons2::goal_width());
     goal_c2l.turn(param_goal_angle);
     param_left_pole_loc = param_goal_loc + goal_c2l;
     param_right_pole_loc = param_goal_loc - goal_c2l;
     goal = -goal_c2l - goal_c2l;
     return 1;
}

Vector Environment::param_ball_loc;
Vector Environment::param_ball_vel;
Vector Environment::param_ball_start_loc;
Vector Environment::param_ball_prev_loc;
Vector Environment::param_team1_loc;
Vector Environment::param_team1_vel;
Vector Environment::param_team2_loc;
Vector Environment::param_team2_vel;
Vector Environment::param_oppo1_loc;
Vector Environment::param_oppo1_vel;
Vector Environment::param_oppo2_loc;
Vector Environment::param_oppo2_vel;

float Environment::param_max_player_coll_time;

Vector Environment::param_defloc;
Vector Environment::param_specloc;
float Environment::param_goal_angle;
Vector Environment::param_goal_loc;
Vector Environment::param_right_pole_loc;
Vector Environment::param_left_pole_loc;
int Environment::param_testmove;
int Environment::param_testkick;
int Environment::param_printvalues;
int Environment::param_endtime;
int Environment::param_viewmap[NUMBER_OF_NAMES];
int Environment::param_calculate[NUMBER_OF_NAMES];


