// -*- mode:C++ -*-
#ifndef ENVIRONMENT_H
#define ENVIRONMENT_H
#include "Vector.h"
#include <v_producer.h>

enum maptypes {BALL, GOAL, DEFLOC, BADSECTOR, NEAR, FAR,
               TEAMMATE, OPPONENT, NO_TEAMMATE, NO_OPPONENT,
               MOVETO_BALL, MOVETO_PLAYER, MOVETO_DEFLOC, MOVETO_SPECLOC,
               MAKE_GOAL, PASS, DRIBBLE, KICKFAR, NUMBER_OF_NAMES};

class Environment
{
public:
     Environment(int exp_no);
     ~Environment();
     update_environment(Vector new_vel, Vector kick, int forward);

     static Vector ball_loc()      { return param_ball_loc;       };
     static Vector ball_vel()      { return param_ball_vel;       };
     static Vector ball_start_loc(){ return param_ball_start_loc; };
     static Vector team1_loc()     { return param_team1_loc;      };
     static Vector team1_vel()     { return param_team1_vel;      };
     static Vector team2_loc()     { return param_team2_loc;      };
     static Vector team2_vel()     { return param_team2_vel;      };
     static Vector oppo1_loc()     { return param_oppo1_loc;      };
     static Vector oppo1_vel()     { return param_oppo1_vel;      };
     static Vector oppo2_loc()     { return param_oppo2_loc;      };
     static Vector oppo2_vel()     { return param_oppo2_vel;      };
     static float  max_player_coll_time()     { return param_max_player_coll_time; };
     static Vector defloc()        { return param_defloc;         };
     static Vector specloc()       { return param_specloc;        };
     static Vector goal_loc()      { return param_goal_loc;       };
     static Vector right_pole_loc(){ return param_right_pole_loc; };
     static Vector left_pole_loc() { return param_left_pole_loc;  };
     static int testmove()         { return param_testmove;       };
     static int testkick()         { return param_testkick;       };
     static int printvalues()      { return param_printvalues ;   };
     static int endtime()          { return param_endtime;      };
     static int viewmap(maptypes wanted) { return param_viewmap[wanted]; };
     static int calculate(maptypes wanted) { return param_calculate[wanted]; };

     void enable_make_goal(){ param_calculate[MAKE_GOAL] = 1; };
     void enable_pass(){ param_calculate[PASS] = 1; };
     void enable_dribble(){ param_calculate[DRIBBLE] = 1; };
     void enable_farkick(){ param_calculate[KICKFAR] = 1; };
     void disable_make_goal(){ param_calculate[MAKE_GOAL] = 0; };
     void disable_pass(){ param_calculate[PASS] = 0; };
     void disable_dribble(){ param_calculate[DRIBBLE] = 0; };
     void disable_farkick(){ param_calculate[KICKFAR] = 0; };

     void set_oppo1_vel(Vector new_vel){ param_oppo1_vel = new_vel;};
     void set_team1_vel(Vector new_vel){ param_team1_vel = new_vel;};
     
      // static int ()     { return ;      };
     int visualize();
     v_producer *vis;
     
protected:
     
     // Visualisointi:
     const int WIN_WIDTH_IN_PIXS = 500; // ikkunan koko
     const float CS_MAX_VAL = 20.05;    // Maximi etisyys, joka kuvassa nkyy
     const float CS2PIX = WIN_WIDTH_IN_PIXS/(2.0*CS_MAX_VAL); // how many pixels is one unit
     const int FIRST_FREE_ID = 1;      // id's for dynamic objects drawn in the window
     float PLAYER_RAD;
     float BALL_RAD;
     float GOAL_WIDTH;
     
     
     int max_dyn_obj_id;
     int stat_obj_id;
     int pnt_id;
     
     int read_config_file(int cfg_no);
     Vector goal;
     Vector my_vel;
     static Vector param_ball_loc;
     static Vector param_ball_vel;
     static Vector param_ball_start_loc;
     static Vector param_ball_prev_loc;
     static Vector param_team1_loc;
     static Vector param_team1_vel;
     static Vector param_team2_loc;
     static Vector param_team2_vel;
     static Vector param_oppo1_loc;
     static Vector param_oppo1_vel;
     static Vector param_oppo2_loc;
     static Vector param_oppo2_vel;

     static float param_max_player_coll_time; // longer times are not considered
     
     static Vector param_defloc;
     static Vector param_specloc;
     static float param_goal_angle;
     static Vector param_goal_loc;
     static Vector param_right_pole_loc;
     static Vector param_left_pole_loc;
     static int param_testmove;
     static int param_testkick;
     static int param_printvalues;
     static int param_endtime;
     static int param_viewmap[NUMBER_OF_NAMES];
     static int param_calculate[NUMBER_OF_NAMES];
};

#endif
