// Advanced Actionmap test program
// Author: Antti Tikanmki
// 
// NOTE:
// distances in meters,
// speeds in meter/0.1 sec,
// angles in radians
#include <Vector.h>
#include <Am/Polargrid.h>
#include <sys/time.h>
#include <iostream.h>
#include <v_producer.h>

#include <Am/Moveto_maps.h>
#include <Am/Kickto_Opp_map.h>
#include <Am/Kickto_Teammate_map.h>
#include <Am/Kickto_Near_map.h>
#include <Am/Kickto_Far_map.h>
#include <Am/Kickto_Goal_map.h>
#include <Am/Kickto_Bad_sector_map.h>
#include <Am/Kickto_Far_map.h>

#include <Const.h>
#include <Cons2.h>
#include <Environment.h>

main(int argc, char *argv[])
{
    if(argc != 2) { 
        cerr << "Usage: program experiment_number" << endl; 
        exit(1);
    }
     cout.precision(3);
     int exp_no = (int) atof(argv[1]);
     cout << "EXPERIMENT NUMBER " << exp_no << endl;

     while(1){
         
         Cons2 cons2;
         Environment environment(exp_no);
         // Karttojen luonti
         Moveto_Ball_map     mballmap;
         //Moveto_Avoid_ball_map mavoidballmap;
         Moveto_Avoid_ball_far_map mavoidballmap;
         Moveto_Player_map   mteam1map;
         Moveto_Player_map   mteam2map;
         Moveto_Player_map   moppo1map;
         Moveto_Player_map   moppo2map;
         Moveto_Location_map   mdeflocmap;
         Moveto_Location_map mspeclocmap;
         Moveto_Ball_map     oppo1_mballmap;
         Moveto_Ball_map     team1_mballmap;
         
         Kickto_Opp_map      koppo1map;
         Kickto_Opp_map      koppo2map;
         Kickto_Teammate_map kteam1map;
         Kickto_Teammate_map kteam2map;
         Kickto_Near_map     knearmap;
         Kickto_Far_map      kfarmap;
         Kickto_Goal_map     kgoalmap;
         Kickto_Bad_sector_map kbadsectmap;
         Moveto_map movemap;
         Moveto_map moveteammap;
         Moveto_map moveoppmap;
         Moveto_map avoidteammap;
         Moveto_map avoidoppmap;
         Moveto_map moveplayersmap;
         Kickto_map kickmap;
         Kickto_map kopponentsmap;
         Kickto_map kteammatesmap;
         
         Vector my_vel(0.0,0.0);
         Vector kick(0.0,0.0);
         Vector best_kick(0.0,0.0);
         Vector best_move(0.0,0.0);
         
         Vector temp(0.0,0.0);
         char cmd_char;
         float end_time = environment.endtime();
         static Vector null_velocity;
         
         char title2[200], eps_title[200];     

         for(int time = 0; time<end_time;time++){
             environment.update_environment(my_vel, kick, 1);
             kick.set_dir_len(0.0,0.0);
             my_vel.set_dir_len(0.0,0.0);
             
             // Kick loop
             if(environment.testkick() 
                && environment.ball_loc().get_len() <= 2.0 ){
                 cerr<<"I AM KICKING!"<<endl;
                 kopponentsmap.reset();
                 kteammatesmap.reset();
                 koppo1map.calculate_map(environment.ball_loc(), environment.ball_vel(),
                                         environment.oppo1_loc(), environment.oppo1_vel());
                 koppo2map.calculate_map(environment.ball_loc(), environment.ball_vel(),
                                         environment.oppo2_loc(), environment.oppo2_vel());
                 kteam1map.calculate_map(environment.ball_loc(), environment.ball_vel(),
                                         environment.team1_loc(), environment.team1_vel());
                 kteam2map.calculate_map(environment.ball_loc(), environment.ball_vel(),
                                         environment.team2_loc(), environment.team2_vel());
                 kgoalmap.calculate_map (environment.ball_loc(), environment.ball_vel(),
                                         environment.goal_loc(),
                                         environment.left_pole_loc(), environment.right_pole_loc());
                 knearmap.calculate_map (environment.ball_loc(), environment.ball_vel(),
                                         environment.goal_loc());
                 kfarmap.calculate_map(environment.ball_loc(), environment.ball_vel(),
                                       environment.goal_loc());
                 kbadsectmap.calculate_map(environment.ball_loc(), environment.ball_vel());
                 
                 kopponentsmap << koppo1map;
                 kopponentsmap << koppo2map;
                 kteammatesmap << kteam1map;
                 kteammatesmap << kteam2map;
                 
                 if(environment.calculate(MAKE_GOAL)){
                     cerr<<"MAKING_GOAL"<<endl;
                     kickmap.reset();
                     kickmap +=kgoalmap;
                     //kopponentsmap.invert();
                     //kickmap -=kteammatesmap;
                     kickmap -= kopponentsmap;
                     kickmap.use_invmask(kbadsectmap); 
                     if(!kickmap.find_best_action(best_kick))
                         environment.enable_dribble();
                     else environment.disable_dribble();
                 }
                 if(environment.calculate(PASS)){
                     cerr<<"PASS"<<endl;
                     kickmap.reset();
                     kickmap +=kteammatesmap;
                     kopponentsmap.invert();
                     kickmap << kopponentsmap;
                     kickmap.use_invmask(kbadsectmap); 
                     kickmap.find_best_action(best_kick);
                 }
                 
                 if(environment.calculate(DRIBBLE)){
                     cerr<<"DRIBBLE"<<endl;
                     kickmap.reset();
                     kickmap +=knearmap;
                     kickmap -=kteammatesmap;
                     kickmap -=kopponentsmap;
                     kickmap.use_invmask(kbadsectmap);
                     kickmap.find_best_action(best_kick);
                 }
                 if(environment.calculate(KICKFAR)){
                     cerr<<"KICKFAR"<<endl;
                     kickmap.reset();
                     kickmap +=kfarmap;
                     //kickmap -=kteammatesmap;
                     kickmap -=kopponentsmap;
                     kickmap.use_invmask(kbadsectmap); 
                     kickmap.find_best_action(best_kick);
                 }
                 if(environment.ball_loc().get_len()<2.0){
                     if(environment.viewmap(BADSECTOR))kbadsectmap.cpp_send("KICKTO BAD SECTOR");
                     if(environment.viewmap(GOAL))kgoalmap.cpp_send("KICKTO GOAL");
                     if(environment.viewmap(FAR))knearmap.cpp_send("KICKTO FAR");
                     if(environment.viewmap(NEAR))knearmap.cpp_send("KICKTO NEAR");
                     if(environment.viewmap(TEAMMATE))kteammatesmap.cpp_send("KICKTO TEAMMATES_MAP");
                     if(environment.viewmap(OPPONENT)) kopponentsmap.cpp_send("KICKTO OPPONENTS_MAP");
                     if(environment.viewmap(MAKE_GOAL)) kickmap.cpp_send("MAKEGOAL"); 
                     if(environment.viewmap(DRIBBLE)) kickmap.cpp_send("DRIBBLE"); 
                     if(environment.viewmap(PASS)) kickmap.cpp_send("PASS_MAP");
                     if(environment.viewmap(KICKFAR)) kickmap.cpp_send("KICKFAR");
                 }
                 kick = best_kick;
             }
             else if(environment.testmove()
                     && environment.ball_loc().get_len() > Cons2::player_size() ){
                 cerr<<"I AM MOVING!"<<endl;
                 moveteammap.reset();
                 moveoppmap.reset();
                 avoidteammap.reset();
                 avoidoppmap.reset();
                 moveplayersmap.reset();
                 mballmap.calculate_map( environment.ball_loc(), 
                                         environment.ball_vel());
                 mavoidballmap.calculate_map( environment.ball_loc(), 
                                              environment.ball_vel());
                 mteam1map.calculate_map(environment.team1_loc(), 
                                         environment.team1_vel());
                 mteam2map.calculate_map(environment.team2_loc(), 
                                         environment.team2_vel());
                 moppo1map.calculate_map(environment.oppo1_loc(), 
                                         environment.oppo1_vel());
                 moppo2map.calculate_map(environment.oppo2_loc(), 
                                         environment.oppo2_vel());
                 mdeflocmap.calculate_map(environment.defloc(),null_velocity);
                 mspeclocmap.calculate_map(environment.specloc(),null_velocity);

                 mteam1map.filter_times_longer_than(environment.max_player_coll_time());
                 mteam1map.filter_times_longer_than(environment.max_player_coll_time());
                 moppo1map.filter_times_longer_than(environment.max_player_coll_time());
                 moppo2map.filter_times_longer_than(environment.max_player_coll_time());

                 moveteammap << mteam1map;
                 moveteammap << mteam2map;
                 moveoppmap << moppo1map;
                 moveoppmap << moppo2map;
                 avoidteammap -= mteam1map;
                 avoidteammap -= mteam2map;
                 avoidoppmap -= moppo1map;
                 avoidoppmap -= moppo2map;
                 moveplayersmap << mteam1map;
                 moveplayersmap << mteam2map;
                 moveplayersmap << moppo1map;
                 moveplayersmap << moppo2map;
               
                 if(environment.calculate(MOVETO_BALL)){ 
                     movemap.reset();
                     movemap += mballmap;
                     movemap -=moveplayersmap; 
                     movemap.find_best_action(best_move);
                 }
                 else if(environment.calculate(MOVETO_PLAYER)){
                      movemap.reset();
                      movemap +=moveplayersmap; 
                      movemap.find_best_action(best_move);
                 }
                 if(environment.calculate(MOVETO_DEFLOC)){
                     movemap.reset();
                     movemap +=mdeflocmap;
                     movemap -=moveplayersmap; 
                     movemap.find_best_action(best_move);
                 }
                 if(environment.calculate(MOVETO_SPECLOC)){
                     movemap.reset();
                     movemap +=mspeclocmap;
                     movemap -=mavoidballmap;
                     movemap -= moveplayersmap;
                     movemap.filter_speeds_faster_than(1.0);
                     movemap.find_best_action(best_move);
                 }
                 my_vel = best_move;
                 
                 if(environment.viewmap(BALL)) {
                      sprintf(title2,"[%d] Ball loc(%.2f, %.2f) vel(%.2f, %.2f)", 
                              time, environment.ball_loc().get_dir(), environment.ball_loc().get_len(),
                              environment.ball_vel().get_dir(), environment.ball_vel().get_len());
                      mballmap.cpp_send(title2);
                      sprintf(eps_title,"data/mballmap%d.eps", time);
                      mballmap.cpp_send("", eps_title);
                 }

                 if(environment.viewmap(TEAMMATE)) {
                      sprintf(title2,"[%d] Move to Team-mates", time);
                      moveteammap.cpp_send(title2);
                      sprintf(eps_title,"data/moveteammap%d.eps", time);
                      moveteammap.cpp_send("", eps_title);
                 } 

                 if(environment.viewmap(NO_TEAMMATE)) {
                      sprintf(title2,"[%d] Avoid Team-mates", time);
                      avoidteammap.cpp_send(title2);
                      sprintf(eps_title,"data/avoidteammap%d.eps", time);
                      avoidteammap.cpp_send("", eps_title);
                 } 

                 if(environment.viewmap(OPPONENT)) {
                      sprintf(title2,"[%d] Move to Opponents", time);
                      moveoppmap.cpp_send(title2);
                      sprintf(eps_title,"data/moveoppmap%d.eps", time);
                      moveoppmap.cpp_send("", eps_title);
                 } 

                  if(environment.viewmap(NO_OPPONENT)) {
                      sprintf(title2,"[%d] Avoid Opponents", time);
                      avoidoppmap.cpp_send(title2);
                      sprintf(eps_title,"data/avoidoppmap%d.eps", time);
                      avoidoppmap.cpp_send("", eps_title);
                 } 

                 if(environment.viewmap(DEFLOC)) {
                      sprintf(title2,"[%d] Defloc(%.2f, %.2f)", 
                              time, environment.defloc().get_dir(), environment.defloc().get_len());
                      mdeflocmap.cpp_send(title2);
                      sprintf(eps_title,"data/mdeflocmap%d.eps", time);
                      mdeflocmap.cpp_send("", eps_title);
                 }

                 if(environment.viewmap(MOVETO_BALL)) {
                      sprintf(title2,"[%d] MovetoBall", time);
                      movemap.cpp_send(title2);
                      sprintf(eps_title,"data/movemap%d.eps", time);
                      movemap.cpp_send("", eps_title);
                 }

                 if(environment.viewmap(MOVETO_PLAYER)) {
                      sprintf(title2,"[%d] MovetoPlayers", time);
                      moveplayersmap.cpp_send(title2);
                      sprintf(eps_title,"data/moveplayersmap%d.eps", time);
                      moveplayersmap.cpp_send("", eps_title);
                 }

                 if(environment.viewmap(MOVETO_DEFLOC)) movemap.cpp_send("MOVETO_DEFLOC_MAP");
                 if(environment.viewmap(MOVETO_SPECLOC)) movemap.cpp_send("MOVETO_SPECLOC_MAP");
                 //if(environment.viewmap()) mmap.cpp_send("_MAP");    
             } else {
                  cerr << "I AM RESTING!" <<endl;
             }
        
             cout <<"("<<time<<")"<< "(press n or q) " << flush;
             cin >> cmd_char;
             if(cmd_char == 'r'){
                 break;
             }
             
             if(cmd_char == 'q') return 0;
         
             
/*               
                 // Opponents active moving:
                 temp = environment.ball_vel();
                 float angle =  environment.ball_vel().angle_from(environment.oppo1_vel());//.get_dir()
                 temp.turn(angle);
                 Vector oppo1_ball_location;
                 oppo1_ball_location = environment.ball_loc()- environment.oppo1_loc();
                 oppo1_ball_location.turn(angle);
                 oppo1_mballmap.calculate_map(oppo1_ball_location, temp);
                 oppo1_mballmap.find_best_action(temp);
                 cerr<<"opponent moving"; temp.err_print_dir_len();cerr<<endl;
                 temp.turn(-angle);
                 // limit the opponents speed
                 if(temp.get_len()>0.7)temp.set_dir_len(temp.get_dir(),0.5);
                 environment.set_oppo1_vel(temp);
                 
                 // Teammates active moving:
                 temp = environment.ball_vel();
                 angle =  environment.ball_vel().angle_from(environment.team1_vel());
                 temp.turn(angle);
                 Vector team1_ball_location;
                 team1_ball_location = environment.ball_loc()- environment.team1_loc();
                 team1_ball_location.turn(angle);
                 team1_mballmap.calculate_map(team1_ball_location, temp);
                 team1_mballmap.find_best_action(temp);
                 cerr<<"team1 moving"; temp.err_print_dir_len();cerr<<endl;
                 temp.turn(-angle);
                 environment.set_team1_vel(temp);*/
         }

     }
     return 0;     
}

     
     
