// file    : v_obj.cc
// includes: functions for the v_obj class for visualized objects


#include "v_obj.h"


v_obj::v_obj()
{
     active = false;
     type = V_OBJ_CIRC;
     x = 0;
     y = 0;
     dir = 0;
     dim = 0;
     name[0] = '\0';
     clr = V_BLACK;

     // cout << "v_obj - constructor executed" << endl;
}


v_obj::~v_obj()
{
     // cout << "v_obj - destructor executed" << endl;
}


v_obj& v_obj::operator=(const v_obj& obj)
{
     if(this != &obj) {
          active = obj.active;
          type = obj.type;
          x = obj.x;
          y = obj.y;
          dir = obj.dir;
          dim = obj.dim;
          strncpy (name, obj.name, V_MAX_NAME_LEN);
          name[V_MAX_NAME_LEN-1] = '\0';
          clr = obj.clr;
     }
     return *this; //???
}


void v_obj::set(bool oactive, v_obj_type otype, float ox, float oy,
                float odir, float odim,  char *oname, v_color oclr)
{
     active = oactive;
     type = otype;
     x = ox;
     y = oy;
     dir = odir;
     dim = odim;

     set_name(oname);
     //cout << "Put - passed name <" << oname << ">, stored <" << name << ">" << endl;

     clr = oclr;
}
     

void v_obj::get(bool& oactive, v_obj_type& otype, float& ox, float& oy,
                float& odir, float& odim, char *oname, v_color& oclr)
{
     oactive = active;
     otype = type;
     ox = x;
     oy = y;
     odir = dir;
     odim = dim;

     get_name(oname);
     //cout << "Get - stored name <" << name << ">, returned <" << oname << ">" << endl;

     oclr = clr;
}


void v_obj::set_name(char *to)
{
     strncpy (name, to, V_MAX_NAME_LEN);
     name[V_MAX_NAME_LEN-1] = '\0';
}


void v_obj::get_name(char *got)
{
     strncpy (got, name, V_MAX_NAME_LEN); 
     got[V_MAX_NAME_LEN-1] = '\0';
}


// draw() draws the requested object and resets the active flag

void v_obj::draw(window* win)
{
     float half_len=0, x_diff=0, y_diff=0;

     //cout << "Draw - <" << name << ">, color= <" << clr << ">" << endl;

     if(active) {
          switch(type) {
          case V_OBJ_CIRC:
               win->draw_circle(x, y, dim, clr);
               if(name[0] != '\0')
                    win->draw_text(x+dim, y+dim, string(name), clr);
               break;

          case V_OBJ_DISC:
               win->draw_disc(x, y, dim, clr);
               if(name[0] != '\0')
                    win->draw_text(x+dim, y+dim, string(name), clr);
               break;
         
          case V_OBJ_LSEG:
               half_len = dim/2.0;
               x_diff = half_len * cos(dir);
               y_diff = half_len * sin(dir);
               win->draw_segment(x + x_diff, y + y_diff,
                                 x - x_diff, y - y_diff,
                                 clr);
               if(name[0] != '\0')
                    win->draw_text(x, y, string(name), clr);
               break;

          case V_OBJ_LINE:
               x_diff = dim * cos(dir);
               y_diff = dim * sin(dir);
               win->draw_segment(x + x_diff, y + y_diff, x, y, clr);
               if( name[0] != '\0')
                    win->draw_text(x, y, name, clr);
               break;

          case V_OBJ_THICK_LSEG:
          {
               int old_w = win->set_line_width(V_THICK_LINE_W);
               half_len = dim/2.0;
               x_diff = half_len * cos(dir);
               y_diff = half_len * sin(dir);
               win->draw_segment(x + x_diff, y + y_diff,
                                 x - x_diff, y - y_diff,
                                 clr);
               if(name[0] != '\0')
                    win->draw_text(x, y, string(name), clr);

               old_w = win->set_line_width(old_w);
          }
               break;

          case V_OBJ_THICK_LINE:
          {
               int old_w = win->set_line_width(V_THICK_LINE_W);
               x_diff = dim * cos(dir);
               y_diff = dim * sin(dir);
               win->draw_segment(x + x_diff, y + y_diff, x, y, clr);
               if( name[0] != '\0')
                    win->draw_text(x, y, name, clr);
               old_w = win->set_line_width(old_w);
          }
               break;

          default:
               break;
          }
          active = false;
     }
}


// FRIENDS

ostream& operator<<(ostream& s, v_obj& v)
{
     int act_val = (v.active)?1:0;

     return s << '(' << act_val << ',' << v.type << ','
              << v.x << ',' << v.y << ',' << v.dir << ','
              << v.dim << ',' << v.name << ',' << v.clr << ')';
}

/* end of file */


